PRESERVE.
SET printback=off.
OMS /DESTINATION VIEWER = NO.

DEFINE !BOOTR(             
               REPS=!TOKENS(1) !DEFAULT(1000)
              /NUM    =!TOKENS(1)
              /DEN    =!TOKENS(1)
              /Alpha =!TOKENS(1) !DEFAULT(0.05) 
	  /MSR =!TOKENS(1) !DEFAULT(COD)  
 	  /SAVE =!TOKENS(1) !DEFAULT(NO)  ).
**********************************************************************************************************************************************************************************
*   Aug 20, 2006.
* 
*   	This program uses both bootstrap and jackknife methods to obtain confidence intervals for a descriptive statistic specified by the user.
*       	It also produces bootstrap and jackknife statistics such as the jackknife estimate of bias and the acceleration.
*
*   	The program applies three bootstrap methods for computing confidence intervals: (1) the normal method, (2) the percentile method,
*       	and (3) the bias-corrected and accelerated (BCa) method.
*
*   	The statistics for which the program could generate bootstrap confidence intervals are: 

*
*		The average absolute deviation (AAD) 
*		The coefficient of dispersion(COD)
*		Minimum (MIN) 
*		Maximum (MAX) 
*		Mean (MEAN)
*		Median (MEDIAN)
*		The median-centered coefficient of variation	(MDCOV) 
*		The mean-centered coefficient of variation (MNCOV)
*		The price-related differential (PRD)
*		Range (RANGE)
*		Standard deviation (STDDEV) 
*		Weighted mean (WGTMEAN)
*		Variance (Var)
*
*   	The program also allows saving the results of every bootstrap replication and the jackknife values. If the user choose SAVE=YES, the results will
*   	be saved in the following folders, respectively: C:\Temp\DRAWS & C:\Temp\JacknifeValues.
* 
* 	Author:
*       		Benmamoun, Mamoun, Research Analyst, Saint Louis University, Saint Louis, MO, U.S.A.
*           		Please, email me at vmamoun@yahoo.com if you observe any problem or if you have
*           		any comment or question.
*
*   	Acknowledgement & Reference:
*   		The syntax file (C:\Temp\syntax) used to get the number of observations into the jackknife and bootstrap loops was written by Hitchin D. (2001).
*       		"Variance bootstrap spss syntax" upon recommendation from Raynald Levesque. Thanks for both.
*   		Efron & Tibshirani(1993). An Introduction to the Bootstrap.Chapman & Hall.

*
**********************************************************************************************************************************************************************************
* This work is dedicated to my mother and the memory of my beloved and wise father who died few months ago
**********************************************************************************************************************************************************************************.

SET SEED=random.
COMPUTE SRATIO=!NUM / !DEN. 

**************Source of this section: Hitchin (2001) ************************************.
RANK VARIABLES=SRATIO  (A) /N INTO N.
EXECUTE.
DO IF $casenum=1. 
WRITE OUTFILE='C:\Temp\syntax.sps' /'DEFINE !NBCASES() 'N ' !ENDDEFINE.'.
END IF.
EXECUTE.
INCLUDE FILE='C:\Temp\syntax.sps' .

**********************************************************************************************.

SAVE OUTFILE='C:\Temp\Orig_Temp.sav'.
COMPUTE group=1.
COMPUTE gr= $casenum .
SAVE OUTFILE='C:\Temp\Orig_Temp.sav'
 /COMPRESSED.

!IF (!MSR !NE "Var") !THEN
RATIO STATISTICS !NUM  WITH !DEN  BY group  (ASCENDING)
  /MISSING = EXCLUDE
  /OUTFILE('C:\Temp\Theta.sav') = !MSR.
GET  FILE='C:\Temp\Theta.sav'.
RENAME VARIABLES (!MSR=Theta).
!ELSE
RATIO STATISTICS !NUM  WITH !DEN  BY group  (ASCENDING)
  /MISSING = EXCLUDE
  /OUTFILE('C:\Temp\Theta.sav') = STDDEV.
GET  FILE='C:\Temp\Theta.sav'.
RENAME VARIABLES (STDDEV=Theta).
COMPUTE Theta=Theta**2.
EXECUTE.
!IFEND


FILTER OFF.
SELECT IF(ROWTYPE_ ~= "OVERALL").
EXECUTE.
SAVE OUTFILE='C:\Temp\Theta.sav'
 /COMPRESSED.
MATCH FILES/ FILE='C:\Temp\Orig_Temp.sav'
	 / TABLE ='C:\Temp\Theta.sav'
	/ BY N.
SORT CASES BY gr.
SAVE OUTFILE='C:\Temp\CalculTheta.sav'
 /COMPRESSED.

*JACKNIFE.

INPUT PROGRAM .
LOOP SAMPLE=1 to !NBCASES.
  LOOP ID = 1 to !NBCASES.
	COMPUTE gr=ID.
    END CASE.
    LEAVE SAMPLE.
    END LOOP.
END LOOP.
END FILE.
END INPUT PROGRAM .
SORT CASES BY gr.
MATCH FILES/ FILE=*
	 / TABLE ='C:\Temp\CalculTheta.sav'
	/ BY gr.
COMPUTE CASES= $casenum .
COMPUTE elim = !NBCASES * (SAMPLE-1)+SAMPLE .
COMPUTE ONES=1

COMPUTE filter_$=(CASES ~= elim).
VARIABLE LABEL filter_$ 'CASES ~= elim (FILTER)'.
VALUE LABELS filter_$  0 'Not Selected' 1 'Selected'.
FORMAT filter_$ (f1.0).
FILTER BY filter_$.

EXECUTE .

!IF (!MSR !NE "Var") !THEN
RATIO STATISTICS !NUM  WITH !DEN  BY SAMPLE  (ASCENDING)
  /MISSING = EXCLUDE
  /OUTFILE('C:\Temp\JacknifeValues.sav') = !MSR.
GET  FILE='C:\Temp\JacknifeValues.sav'.
RENAME VARIABLES (!MSR=JTheta).
FILTER OFF.
SELECT IF(ROWTYPE_ ~= "OVERALL").
EXECUTE.
!ELSE
RATIO STATISTICS !NUM  WITH !DEN  BY SAMPLE  (ASCENDING)
  /MISSING = EXCLUDE
  /OUTFILE('C:\Temp\JacknifeValues.sav') = STDDEV.
GET  FILE='C:\Temp\JacknifeValues.sav'.
RENAME VARIABLES (STDDEV=JTheta).
COMPUTE JTheta=JTheta**2.
FILTER OFF.
SELECT IF(ROWTYPE_ ~= "OVERALL").
EXECUTE.
!IFEND

AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=ROWTYPE_
  /Theta_mean= MEAN(JTheta).
MATCH FILES/ FILE=*
	 / FILE ='C:\Temp\CalculTheta.sav'.
COMPUTE theta3 = (Theta_mean-JTheta) **3 .
COMPUTE theta2 = (Theta_mean-JTheta) **2 .
AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=ROWTYPE_
  /theta3_sum = SUM(theta3) /theta2_sum = SUM(theta2).

DO IF (theta2_sum NE  0).
COMPUTE den=6* (theta2_sum**1.5). 
COMPUTE accel = theta3_sum / den.
ELSE IF(theta2_sum EQ  0).
COMPUTE accel = 0.
END IF.

COMPUTE JBias = N*(Theta_mean- Theta) .
AGGREGATE
  /OUTFILE='C:\Temp\accel.sav'
  /BREAK=ROWTYPE_
  /accel = FIRST(accel) /JBias = FIRST(JBias).
SAVE OUTFILE='C:\Temp\JacknifeValues.sav'
/KEEP JTheta. 

* BOOTSTRAPING.

INPUT PROGRAM .
LOOP REP=1 to !REPS.
  LOOP N = 1 to !NBCASES.
    COMPUTE gr=TRUNC(UNIFORM(!NBCASES)) + 1.
    END CASE.
    LEAVE REP.
    END LOOP.
END LOOP.
END FILE.
END INPUT PROGRAM .
SORT CASES BY gr .
MATCH FILES / FILE = *
	 / TABLE ='C:\Temp\Orig_Temp.sav'
	/ BY gr.

!IF (!MSR !NE "Var") !THEN
RATIO STATISTICS !NUM  WITH !DEN  BY REP  (ASCENDING)
  /MISSING = EXCLUDE
  /OUTFILE('C:\Temp\DRAWS.sav') = !MSR.
GET
  FILE='C:\Temp\DRAWS.sav'.
RENAME VARIABLES (!MSR=RTheta).
!ELSE
RATIO STATISTICS !NUM  WITH !DEN  BY REP  (ASCENDING)
  /MISSING = EXCLUDE
  /OUTFILE('C:\Temp\DRAWS.sav') = STDDEV.
GET
  FILE='C:\Temp\DRAWS.sav'.
RENAME VARIABLES (STDDEV=RTheta).
COMPUTE RTheta=RTheta**2.
!IFEND.

FILTER OFF.
SELECT IF(ROWTYPE_ ~= "OVERALL").
EXECUTE .
AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=ROWTYPE_
  /Thetase= SD(RTheta) /RTheta_mean = MEAN(RTheta).
MATCH FILES/ FILE=*
	 / TABLE ='C:\Temp\Theta.sav'
	 / TABLE ='C:\Temp\accel.sav'
	/ BY ROWTYPE_.
COMPUTE BBIAS = RTheta_mean-Theta .
COMPUTE Alpha=!Alpha/2.
COMPUTE Alpha2=1-!Alpha.
COMPUTE lb_normal = Theta+ (IDF.NORMAL(Alpha,0,1) * ThetaSE) .
COMPUTE ub_normal = Theta- (IDF.NORMAL(Alpha,0,1) * ThetaSE) .
COMPUTE pcl_obs1=RND(!Alpha*!REPS).
COMPUTE pcl_obs2=RND(Alpha2*!REPS).
SORT CASES BY RTheta.
IF ($CASENUM=pcl_obs1) lb_pcl =RTheta.
IF ($CASENUM=pcl_obs2) ub_pcl =RTheta.
COMPUTE countz0=(RTheta<Theta).
AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=ROWTYPE_
  /countz0_sum = SUM(countz0).
COMPUTE  countless=countz0_sum/!REPS.
COMPUTE z0=IDF.NORMAL(countless,0,1).
COMPUTE z0=IDF.NORMAL(countless,0,1).
COMPUTE z0=IDF.NORMAL(countless,0,1).
COMPUTE  k1=z0+ IDF.NORMAL(!Alpha,0,1).
COMPUTE  k2=z0+ IDF.NORMAL(Alpha2,0,1).
COMPUTE bc1=z0+ k1/(1-accel*k1).
COMPUTE bc2=z0+ k2/(1-accel*k2).
COMPUTE a1=CDF.NORMAL(bc1,0,1).
COMPUTE a2=CDF.NORMAL(bc2,0,1).
COMPUTE bc_obs1=RND(a1*!REPS).
COMPUTE bc_obs2=RND(a2*!REPS).
SORT CASES BY RTheta.
IF ($CASENUM=bc_obs1) lb_bc =RTheta.
IF ($CASENUM=bc_obs2) ub_bc =RTheta.
AGGREGATE
  /OUTFILE='C:\Temp\Stats.sav'
  /BREAK=ROWTYPE_
  /ThetaSE = FIRST(ThetaSE) /Theta = FIRST(Theta) /z0 = FIRST(z0) /JBias = FIRST(JBias) /lb_normal = FIRST(lb_normal) /ub_normal = FIRST(ub_normal)
 /lb_pcl = MAX(lb_pcl) /ub_pcl = MAX(ub_pcl) /a1 = FIRST(a1) /a2 = FIRST(a2) /Alpha2 = FIRST(Alpha2)/lb_bc = MAX(lb_bc) /ub_bc = MAX(ub_bc)/accel= FIRST(accel)/BBias= FIRST(BBias).

SAVE OUTFILE='C:\Temp\DRAWS.sav'
 /KEEP=RTheta Theta. 

OMSEND.

TITLE "Bootstrap & Jackknife Estimates".
GET FILE='C:\Temp\Stats.sav'.
COMPUTE conf=Alpha2*100.
PRINT  SPACE.
PRINT / "Bootstrap Confidence Intervals".
PRINT  SPACE .
PRINT /"                       " 1 "    Normal" 25 "     Percentile" 45 "   BCa" 65.
PRINT  SPACE.
PRINT /"    Lower Bound :" 1   lb_normal(T25, F14.5) lb_pcl(T45, F14.5) lb_bc(T65, F14.5) .
PRINT / "    Upper Bound :" 1   ub_normal(T25, F14.5) ub_pcl(T45, F14.5) ub_bc(T65, F14.5) .
PRINT  SPACE .
PRINT / "Bootstrap & Jackknife Statistics".
PRINT  SPACE.
PRINT /" Observed:" 1 Theta (T25, F14.5). 
PRINT /" Jackknife Bias  :" 1 JBias (T25, F14.5). 
PRINT /" Bootstrap Bias  :" 1 BBias (T25, F14.5). 
PRINT /" Std. Err :" 1 ThetaSE (T25, F14.5). 
PRINT /" Acceleration :" 1 accel (T25, F14.5). 
PRINT /" Median Bias (zo):" 1 z0 (T25, F14.5). 
PRINT /" BCa_Alpha1 :" 1 a1 (T25, F14.5). 
PRINT /" BCa_Alpha2 :" 1 a2 (T25, F14.5). 
EXECUTE.

GET
  FILE='C:\Temp\DRAWS.sav'.

!IF (!MSR !NE "Var") !THEN
GRAPH
  /HISTOGRAM(NORMAL)=RTheta
  /TITLE= 'The Bootstrap Distribution '.
!ELSE
VARIABLE LABELS RTheta 'Variance'.
GRAPH
  /HISTOGRAM(NORMAL)=RTheta
  /TITLE= 'The Bootstrap Distribution '.
!IFEND


GET FILE='C:\Temp\Orig_Temp.sav'
/DROP= N group gr SRATIO.
SAVE OUTFILE='C:\Temp\Orig_Temp.sav'
 /COMPRESSED.

!IF (!SAVE = 'NO')!then
ERASE  FILE='C:\Temp\accel.sav'.
ERASE  FILE='C:\Temp\CalculTheta.sav'.
ERASE  FILE='C:\Temp\JacknifeValues.sav'.
ERASE  FILE='C:\Temp\Stats.sav'.
ERASE  FILE='C:\Temp\Theta.sav'.
ERASE  FILE='C:\Temp\DRAWS.sav'.
!ELSE
ERASE  FILE='C:\Temp\accel.sav'.
ERASE  FILE='C:\Temp\CalculTheta.sav'.
ERASE  FILE='C:\Temp\Stats.sav'.

!IFEND

RESTORE.
!ENDDEFINE.








