PRESERVE.
SET printback=off.
OMS /DESTINATION VIEWER = NO.

**********************************************************************************************************************************************************************
* IMPORTANT (Please Read) : 
*		Your use of this program is at your sole risk. Under no circumstances, shall the author and the owner of this program 
*		be liable for any direct or indirect losses or damages resulting from the use or inability to use this program. 
**********************************************************************************************************************************************************************

DEFINE !CODCI(             
              VAR    =!TOKENS(1)
              /ALPHA =!TOKENS(1) !DEFAULT(0.05)). 

*****************************************************************************************************************************************************************
*  	September 29, 2006.
*
*	This program computes the confidence interval for a coefficient of dispersion in nonnormal distributions as poposed . 
*		by Bonnett & Seier (2006) 
*  	
*	Syntax diagram:
*
* 		!CODCI varname 
* 		[ALPHA={0.05}]
*       			{value}
*
*	varname: the user MUST specify the name of the variable on which the command is to be performed. 
*	APLHA= #.The user could specify the level of significance. This is optional. The default is 0.05. 
*
* 	Example: Using the dataset from Bonnett & Seier (2006, p. 146), we would like to obtain an approximate 
*		95% confidence interval for the COD .
*
*	Output is:
*	
*	Bonett-Seier's COD Confidence Interval
*
* 	--------------------------------------------------
* 	Coefficient of Dispersion (COD) :        .44444
*	--------------------------------------------------
*	Confidence Interval for the COD
*
*    		Lower Bound :                      .29185
*   		Upper Bound :                     1.06570
*
*	--------------------------------------------------
*
*
*	Author:
*		Benmamoun, Mamoun, Research Analyst, Saint Louis University, Saint Louis, MO, U.S.A. 
*			Please, email me at vmamoun@yahoo.com if you observe any problem or if you have 
*			any comment or question.
*	Reference: 
*		-Bonett, D. G. and Seier, E. (2006). Confidence Interval for a Coefficient of Dispersion in Nonnormal Distributions
*			 Biometrical Journal 48, 144-148
*	
**********************************************************************************************************************************************************************

SAVE OUTFILE='C:\Temp\Orig_Temp.sav'.
COMPUTE break = 1 .
EXECUTE .
AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=break
  /eta = MEDIAN(!VAR)
  /u = MEAN(!VAR)
  /sd = SD(!VAR)
  /n=N.
COMPUTE v = sd ** 2 .
EXECUTE .
COMPUTE abs = ABS(!VAR-eta) .
EXECUTE .
AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=break
  /tau = MEAN(abs).
COMPUTE del = (u-eta)/tau .
EXECUTE .
COMPUTE gam = v/( tau ** 2) .
EXECUTE .
COMPUTE cod = tau / eta .
EXECUTE .
RANK VARIABLES=!VAR  (A)  /RANK INTO ORDERS /TIES=MEAN.
COMPUTE astar = RND((n+1)/2-SQRT(n) ) .
EXECUTE .
COMPUTE bstar = n-astar+1 .
EXECUTE .
IF (astar=ORDERS) aorder = !VAR .
EXECUTE .
IF (bstar=ORDERS) border = !VAR .
EXECUTE .
AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=break
  /a_order = MAX(aorder) /b_order = MAX(border).
COMPUTE varleta = ((LN(a_order)-LN(b_order))/4) ** 2 .
EXECUTE .
COMPUTE se1 = sqrt(varleta) .
EXECUTE .
COMPUTE varltau = (gam+ (del ** 2)-1) / n .
EXECUTE .
COMPUTE se2 = sqrt(varltau) .
EXECUTE .
COMPUTE covltle = (del*sqrt(varleta))/sqrt(n) .
COMPUTE alpha2 = 1-(!ALPHA/2).
EXECUTE .
COMPUTE k = sqrt (varleta + varltau - (2*covltle))/(se1 + se2) .
EXECUTE .
COMPUTE a = RND(((n+1)/2)-(k*sqrt(n/4)* IDF.NORMAL(alpha2,0,1))) .
EXECUTE .
COMPUTE b = n-a+1 .
EXECUTE .
IF (ORDERS = a) a2order = !VAR .
EXECUTE .
IF (ORDERS = b) b2order = !VAR .
EXECUTE .
AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=break
  /a_2order = MAX(a2order) /b_2order = MAX(b2order).
COMPUTE L2star = LN(a_2order) .
EXECUTE .
COMPUTE U2star = LN(b_2order) .
EXECUTE .
COMPUTE L1 = LN((n/(n-1))*tau)- (k*IDF.NORMAL(alpha2,0,1)*se2) .
EXECUTE .
COMPUTE U1 = LN((n/(n-1))*tau)+(k*IDF.NORMAL(alpha2,0,1)*se2) .
EXECUTE .
COMPUTE Lower = EXP(L1-U2star) .
EXECUTE .
COMPUTE Upper = EXP(U1-L2star) .
EXECUTE .

SAVE OUTFILE='C:\Temp\Orig_Temp.sav'
/KEEP= !VAR cod Lower Upper .

OMSEND.

DO IF $CASENUM EQ 1.
TITLE "Bonett-Seier's COD Confidence Interval".
PRINT   / "--------------------------------------------------".
PRINT /" Coefficient of Dispersion (COD) :" 1 cod (T35, F14.5).
PRINT   / "--------------------------------------------------". 
PRINT / " Confidence Interval for the COD".
PRINT  SPACE .
PRINT / "    Lower Bound :" 1   Lower (T35, F14.5) .
PRINT / "    Upper Bound :" 1   Upper (T35, F14.5) .
PRINT  SPACE .
PRINT   / "--------------------------------------------------".

END IF.
EXECUTE.

GET FILE='C:\Temp\Orig_Temp.sav'.

RESTORE.
!ENDDEFINE.




