PRESERVE.
SET printback=off.
OMS /DESTINATION VIEWER = NO.

**********************************************************************************************************************************************************************
* IMPORTANT (Please Read) : 
*		Your use of this program is at your sole risk. Under no circumstances, shall the author and the owner of this program 
*		be liable for any direct or indirect losses or damages resulting from the use or inability to use this program. 
**********************************************************************************************************************************************************************

DEFINE !CODTEST(             
              VAR    =!TOKENS(1)
              /STDR    =!TOKENS(1) !DEFAULT(15)
              /ALPHA =!TOKENS(1) !DEFAULT(0.05)). 

*****************************************************************************************************************************************************************
*  	July 05, 2006.
*
*	This program computes the confidence interval for the COD, the maximum acceptable COD, 
*		the tolerance factor, and tests the hypothesis that the COD meet the required 
*		or the target standard.
*
*  	Syntax diagram:
*
* 		!CODTEST varname 
* 		[STDR ={15}]
*	  		{value}
* 		[ALPHA={0.05}]
*       			{value}
*
*	varname: the user MUST specify the name of the variable on which the command is to be performed. 
*	STDR= #. The user could set the value of the required or the target standard. This is optional. The default is 15. 
*	APLHA= #.The user could specify the level of significance. This is optional. The default is 0.05. 
*
* 	Example: From the Case Study of New York Towns described in Gloudemans( 2001). "Confidence
*		Intervals for the COD: Limitations and Solutions."  Available online at 
*		http://www.agjd.com/COD.pdf. With a significance of 0.05 (default), we are interested 
*		to test whether the COD meet a specific standard of 15% (default).
*
*   	Typing: !CODTEST  VAR=ratio STDR=20 ALPHA=0.05.
*
*	Output is:
*
*	Gloudemans' COD Tolerance Test
*
*		----------------------------------------------------------------------------------------------------
*		Confidence Interval for the COD**
*		
*		 Lower Bound :                          			14.59379
*		 Upper Bound :                          			24.63395
*		
*		----------------------------------------------------------------------------------------------------
*		Ho: COD # CODSTD
*		
*		 Coefficient of Dispersion (%) :        		18.38308
*		 Degree of freedom:                     		29.00000
*		 Chi-square value :                    	 		43.55643
*		 Critical value :                      		 	42.55697
*		 P-value(1-tail)  :                       			   .04041
*		 Tolerance Factor (TF):                 	 	 1.21140
*		 Maximum Acceptable COD (MAXCOD):             18.17095
*		----------------------------------------------------------------------------------------------------
*		**Assume Normality
* 	Author:
*		Benmamoun, Mamoun, Research Analyst, Saint Louis University, Saint Louis, MO, U.S.A. 
*			Please, email me at vmamoun@yahoo.com if you observe any problem or if you have 
*			any comment or question.
**	Note: 
*		-This program is based exclusively on the distinguished work of Gloudemans (2001).
*	
**********************************************************************************************************************************************************************
* This work is dedicated to my mother and the memory of my beloved and wise father who died few months ago
***********************************************************************************************************************************************************************.

SAVE OUTFILE='C:\Temp\Orig_Temp.sav'.
COMPUTE group=1.
COMPUTE UNO = 1 .
AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=group
  /ratio_mean = MEAN(!VAR) /ratio_median = MEDIAN(!VAR) /ratio_sum = SUM(!VAR) /ratio_sd = SD(!VAR)
  /N=N.
SAVE OUTFILE='C:\Temp\Orig_Temp.sav'
 /COMPRESSED.

RATIO STATISTICS !VAR  WITH UNO  BY group  (ASCENDING)
  /MISSING = EXCLUDE
  /OUTFILE('C:\Temp\cod.sav') =COD.
GET  FILE='C:\Temp\cod.sav'.
RENAME VARIABLES (COD=cod).
FILTER OFF.
SELECT IF(ROWTYPE_ ~= "OVERALL").
EXECUTE.
SAVE OUTFILE='C:\Temp\cod.sav'
 /COMPRESSED.

MATCH FILES/ FILE='C:\Temp\Orig_Temp.sav'
	 / TABLE ='C:\Temp\cod.sav'
	/ BY N.

COMPUTE p1=!ALPHA/2.
COMPUTE p2=1-!ALPHA/2.
COMPUTE p3=1-!ALPHA.
COMPUTE df=N-1.
COMPUTE cisd_ub=Sqrt((df*ratio_sd**2)/IDF.CHISQ(p1, df)).
COMPUTE cisd_lb=Sqrt((df*ratio_sd**2)/IDF.CHISQ(p2, df)).
COMPUTE cicov_ub=100*cisd_ub/ratio_mean.
COMPUTE cicov_lb=100*cisd_lb/ratio_mean.
COMPUTE cicod_ub=cicov_ub*0.80.
COMPUTE cicod_lb=cicov_lb*0.80.
COMPUTE chi2cod = df*cod**2/!STDR**2.
COMPUTE ctail = IDF.CHISQ(p3,df).
COMPUTE tf=Sqrt(ctail/df).
COMPUTE maxcod= !STDR* tf.
COMPUTE pu = SIG.CHISQ(chi2cod, df).
COMPUTE pl = 1-SIG.CHISQ(chi2cod, df).
COMPUTE prob=min(2*pl, 2*pu).
SAVE OUTFILE='C:\Temp\Orig_Temp.sav'
/DROP=group UNO ratio_mean ratio_median ratio_sum ratio_sd N ROWTYPE_ 
cod p1 p2 p3 df cisd_ub cisd_lb cicov_ub cicov_lb cicod_ub 
cicod_lb chi2cod  ctail tf  maxcod pu pl prob.

OMSEND.

DO IF $CASENUM EQ 1.
TITLE "Gloudemans' COD Tolerance Test".
PRINT   / "--------------------------------------------------".
PRINT / "Confidence Interval for the COD**".
PRINT  SPACE .
PRINT /" Lower Bound :" 1   cicod_lb(T35, F14.5) .
PRINT / " Upper Bound :" 1   cicod_ub(T35, F14.5) .
PRINT  SPACE .
PRINT   / "--------------------------------------------------".
PRINT / "Ho: COD # CODSTD ".
PRINT  SPACE.
PRINT /" Coefficient of Dispersion (%) :" 1 cod (T35, F14.5). 
PRINT /" Degree of freedom:" 1 df (T35, F14.5). 
PRINT /" Chi-square value :" 1 chi2cod (T35, F14.5). 
PRINT /" Critical value :" 1 ctail (T35, F14.5). 
PRINT /" P-value(1-tail)  :" 1 pu (T35, F14.5). 
PRINT /" Tolerance Factor (TF):" 1 tf (T35, F14.5). 
PRINT /" Maximum Acceptable COD (MAXCOD):" 1 maxcod (T35, F14.5). 
PRINT   / "--------------------------------------------------".
PRINT / "**Assume Normality".
END IF.
EXECUTE.

ERASE  FILE='C:\Temp\cod.sav'.
GET FILE='C:\Temp\Orig_Temp.sav'.
RESTORE.
!ENDDEFINE.




