

DEFINE BSUBSETS(!POS=!TOKENS(1)/!POS=!CMDEND).
CACHE. /* This avoids an error in MATRIX when temporary files /*.
EXE.   /* are used more than once                             /*.
PRESERVE.
SET MXLOOPS=1000000. /* Should be at least equal to 2^k - 1 *.
SET WORKSPACE=200000.
MATRIX.
PRINT /TITLE='              ========================================'.
PRINT /TITLE='              MULTIPLE LINEAR REGRESSION: BEST SUBSETS'.
PRINT /TITLE='              ========================================'.
* Input data (DV goes first) with listwise deletion *.
GET data /VAR=!1 !2/MISSING=OMIT /NAME=vnames.
* Split data in y&x arrays (adding a column of "1" to x to estimate constant) *.
COMPUTE y=data(:,1).
COMPUTE x={MAKE(NROW(data),1,1),data(:,(2:NCOL(data)))}.
* Auxilliary variable (for output labelling in unstandardized coefficients report) *.
COMPUTE xnames=vnames(2:NCOL(data)).
COMPUTE vnames={'Constant',vnames(2:NCOL(data))}.
RELEASE data. /* Get rid of original data array *.
* Compute statistics for full model *.
COMPUTE p=NCOL(x).
COMPUTE n=NROW(x).
COMPUTE meany=CSUM(y)/n.
COMPUTE b=GINV(x)*y.                    /* Note: GINV=INV[T(x)*x]*T(x) *.
COMPUTE TSS=CSSQ(y)-n*meany&**2.
COMPUTE ESS=T(b)*T(x)*y-n*meany&**2.
COMPUTE RSS=TSS-ESS.
COMPUTE sigma2=RSS/(n-p).
COMPUTE SEb=SQRT(DIAG(sigma2*INV(T(x)*x))).
COMPUTE tvalue=b/SEb.
COMPUTE tsig=2*(1-TCDF(ABS(tvalue),n-p)).
COMPUTE rsquare=ESS/TSS.
COMPUTE psquare=1-(1-rsquare)*(n-1)/(n-p).
COMPUTE MallowCp=(RSS/sigma2)+2*p-n.
COMPUTE AIC=n*LN(RSS/n)+2*p.
COMPUTE SBc=n*LN(RSS/n)+p*LN(n).
COMPUTE Fvalue=(rsquare/(p-1))/((1-rsquare)/(n-p)).
COMPUTE Fsig=1-FCDF(Fvalue,(p-1),(n-p)).
* Reports for full model *.
PRINT /TITLE='************ STATISTICS FOR FULL MODEL ************'.
PRINT {n,p-1}
 /FORMAT='F8.0'
 /CLABELS='N','K'
 /TITLE='Sample size & number of predictors'.
PRINT {b,SEb,tvalue,tsig}
 /FORMAT='F8.3'
 /RNAMES=vnames
 /CLABELS='b','SE(b)','t value','2-tail P'
 /TITLE='Statistics for unstandardized coefficients (full model)'.
PRINT {rsquare,psquare,SQRT(sigma2)}
 /FORMAT='F8.3'
 /CLABEL='R-Square','P-Square','Res(SD)'
 /TITLE='Sample R-square, Adjusted (Population) Rho-square & Residual SD'.
PRINT {Fvalue,Fsig}
 /FORMAT='F8.4'
 /CLABEL='F value','Sig.'
 /TITLE='F test for R-square'.
*********** COMPUTING INFO FOR EVERY OTHER POSSIBLE MODEL ***********.
* Eliminate 1st column of x array (will be added later inside a loop) *.
COMPUTE subx=x(:,(2:p)).
COMPUTE nx=NCOL(subx).
RELEASE x.
* Create all subset patterns for nx predictors (algorithm by M G-Granero) *.
COMPUTE subset={0;1}.
COMPUTE nsub=NROW(subset).
LOOP i=2 TO nx. /* Expanding array to nx variables by recursive duplication *.
. COMPUTE subset={MAKE(nsub,1,0),subset;MAKE(nsub,1,1),subset}.
. COMPUTE nsub=NROW(subset).
END LOOP.
COMPUTE nvars=RSUM(subset).
* Initialize empty matrix where statistics for every model will be stored *.
COMPUTE totals=NROW(subset). 
COMPUTE allstats=MAKE(totals,6,0).
* Statistics for all possible models *.
LOOP i=1 TO totals.  /* This LOOP needs that MXLOOPS is GE than "totals" *.
. DO IF nvars(i) EQ nx.
* Info for full model is stored here *.
.  COMPUTE allstats(i,1)=rsquare.
.  COMPUTE allstats(i,2)=psquare.
.  COMPUTE allstats(i,3)=SQRT(sigma2).
.  COMPUTE allstats(i,4)=MallowCp.
.  COMPUTE allstats(i,5)=AIc.
.  COMPUTE allstats(i,6)=SBc.
. ELSE IF nvars(i) NE 0.
* Info for every other model *.
* Create x matrix at every step according to "i" subset pattern *.
.  COMPUTE x={MAKE(n,1,1)}. /* Constant is added first *.
.  LOOP j=1 TO nx.
.   DO IF subset(i,j) EQ 1. /* Add variable only if its value in subset is 1 *.
.    COMPUTE x={x,subx(:,j)}.
.   END IF.
.  END LOOP.
* Compute statistics for restricted model "i" *.
.  COMPUTE p=NCOL(x).
.  COMPUTE b=GINV(x)*y.
.  COMPUTE ESS=T(b)*T(x)*y-n*meany&**2.
.  COMPUTE RSS=TSS-ESS.
* Store important statistics in 'allstats' for later *.
.  COMPUTE allstats(i,1)=ESS/TSS.                   /* R-square     *.
.  COMPUTE allstats(i,2)=1-(1-ESS/TSS)*(n-1)/(n-p). /* Rho-square   *.
.  COMPUTE allstats(i,3)=SQRT(RSS/(n-p)).           /* ResSD        *.
.  COMPUTE allstats(i,4)=(RSS/sigma2)+2*p-n.        /* Mallow's Cp  *.
.  COMPUTE allstats(i,5)=n*LN(RSS/n)+2*p.           /* Akaike IC    *.
.  COMPUTE allstats(i,6)=n*LN(RSS/n)+p*LN(n).       /* Schwartz BC  *.
. ELSE. /* This handles the particular case when nvars EQ 0 (empty model) *. 
* Nothing is done because all 6 values are already 0 *.
. END IF.
END LOOP.
* Report of number of models *.
PRINT {totals-1}
 /FORMAT='F8'
 /TITLE='Total number of models computed and exported to AllModels.sav'.
* Export ALL models to another temporary file for further analyses *.
COMPUTE export={subset,allstats}.
COMPUTE xnames={xnames,'R2','Rho2','ResSD','Cp','AIC','SBC'}.
SAVE export /OUTFILE='C:\Temp\AllModels.sav'/NAMES=xnames.
END MATRIX.

RESTORE.
SAVE OUTFILE='C:\Temp\OriginalFile.sav'.

* Read file with all models and start working *.
GET FILE='C:\Temp\AllModels.sav'.
COMPUTE nvars=SUM(!2).
SELECT IF nvars NE 0. /* Get rid of null model *.
COUNT nx = !2 (0) !2 (1)  .
PRESERVE.
SET ERRORS=   NONE
   /RESULTS=  NONE
   /PRINTBACK=NONE
   /MESSAGES= NONE.
RANK  VARIABLES=Cp (A) BY nvars /RANK /PRINT=NO.
RESTORE.
COMPUTE Pprime=nvars+1.
COMPUTE Psec=2*nvars-nx.
FORMAT Pprime Psec(F8) R2 TO SBC (F8.1).
VAR LABEL Pprime '.' /R2 'R' /Rho2 'Adj.R'.
****** GRAPHS FOR BEST MODELS ONLY ***.
* Graph Cp vs Pprime *.
TEMPORARY.
SELECT IF (RCp EQ 1).
GRAPH /SCATTERPLOT(OVERLAY)=pprime WITH Cp pprime psec.
* Graph Adjusted/Unadjusted R-square vs Pprime *.
TEMPORARY.
SELECT IF (RCp EQ 1).
GRAPH /SCATTERPLOT(OVERLAY)=pprime WITH R2 Rho2.
* Graph AIC/SCB vs Pprime *.
TEMPORARY.
SELECT IF (RCp EQ 1).
GRAPH /SCATTERPLOT(OVERLAY)=pprime WITH AIC SBC.
* Listing 5 best models for every number of predictors (thanks to Simon Freidin for his help) *.
FORMAT !2 nvars (F3) R2 TO SBC (F5.3).
VAL LABEL !2 0' ' 1'X'.
SORT CASES BY nvars(A) Cp(A) .
SPLIT FILE LAYERED BY nvars .
OMS /SELECT TABLES
 /IF COMMANDS='Summarize' SUBTYPES='Case Processing Summary'
 /DESTINATION VIEWER=NO.
TEMPORARY.
SELECT IF (RCp LE 5).
SUMMARIZE
  /TABLES=!2 R2 TO sbc
  /FORMAT=LIST NOCASENUM NOTOTAL
  /TITLE='Best subsets models'
  /MISSING=VARIABLE
  /CELLS=NONE.
OMSEND.
SPLIT FILE OFF.
SAVE OUTFILE='C:\Temp\AllModels.sav'.
GET FILE='C:\Temp\OriginalFile.sav'.
!ENDDEFINE.

DEFINE RSUBSET(!POS=!TOKENS(1)/!POS=!CHAREND('/')/file=!CMDEND).

CACHE. /* This avoids an error in MATRIX when temporary files /*.
EXE.   /* are used more than once                             /*.

* Step 1: All possible subsets are created & stored *.
MATRIX.
* Input data (DV goes first) with listwise deletion *.
GET data  /VAR=!1 !2/MISSING=OMIT /NAME=vnames.
* Input data are saved for step 3 *.
SAVE data /OUTFILE='C:\Temp\RegressionData.sav' /NAMES=vnames.
* Predictors variables names are extracted *.
COMPUTE xnames=vnames(2:NCOL(data)).
RELEASE data. /* Get rid of original data *.
* Create all subset patterns for nx predictors (algorithm by M G-Granero) *.
COMPUTE nx=NCOL(xnames).
COMPUTE subset={0;1}.
COMPUTE nsub=NROW(subset).
LOOP i=2 TO nx. /* Expanding array to nx variables by recursive duplication *.
. COMPUTE subset={MAKE(nsub,1,0),subset;MAKE(nsub,1,1),subset}.
. COMPUTE nsub=NROW(subset).
END LOOP.
PRINT NROW(subset)
 /FORMAT='F8.0'
 /TITLE='All possible subsets saved to SubsetsList.sav'.
PRINT /TITLE='User restrictions will be applied now before computing statistics'.
SAVE subset /OUTFILE='C:\Temp\SubsetsList.sav'/NAMES=xnames.
END MATRIX.
SAVE OUTFILE='C:\Temp\OriginalFile.sav'.

* Step 2: User restrictions are applied to subsets *.
GET FILE='C:\Temp\SubsetsList.sav'.
COMPUTE nvars=SUM(!2).
SELECT IF nvars NE 0. /* Get rid of null model *.
***** THE FILE MUST BE CREATED BEFORE CALLING THE MACRO *****.
PRESERVE.
SET PRINTBACK=ON.
INCLUDE !file.
RESTORE.
SORT CASES BY nvars(D).
MATCH FILES/FILE=* /DROP=nvars.
SAVE OUTFILE='C:\Temp\SubsetsList.sav'.

* Step 3: Statistics are computed for every model in the restricted list *.
PRESERVE.
SET MXLOOPS=500000.
SET WORKSPACE=10000.
MATRIX.
PRINT /TITLE='   ========================================================'.
PRINT /TITLE='   MULTIPLE LINEAR REGRESSION: USER RESTRICTED BEST SUBSETS'.
PRINT /TITLE='   ========================================================'.
GET data /VAR=ALL /NAME=vnames /FILE='C:\Temp\RegressionData.sav'.
* Split data in y&x arrays (adding a column of "1" to x for the constant) *.
COMPUTE y=data(:,1).
COMPUTE x={MAKE(NROW(data),1,1),data(:,(2:NCOL(data)))}.
* Auxilliary variable (for output labelling in unstandardized coefficients report) *.
COMPUTE vnames={'Constant',vnames(2:NCOL(data))}.
RELEASE data. /* Get rid of original data array *.
* Compute statistics for full model *.
COMPUTE p=NCOL(x).
COMPUTE n=NROW(x).
COMPUTE meany=CSUM(y)/n.
COMPUTE b=GINV(x)*y.                    /* Note: GINV=INV[T(x)*x]*T(x) *.
COMPUTE TSS=CSSQ(y)-n*meany&**2.
COMPUTE ESS=T(b)*T(x)*y-n*meany&**2.
COMPUTE RSS=TSS-ESS.
COMPUTE sigma2=RSS/(n-p).
COMPUTE SEb=SQRT(DIAG(sigma2*INV(T(x)*x))).
COMPUTE tvalue=b/SEb.
COMPUTE tsig=2*(1-TCDF(ABS(tvalue),n-p)).
COMPUTE rsquare=ESS/TSS.
COMPUTE psquare=1-(1-rsquare)*(n-1)/(n-p).
COMPUTE MallowCp=(RSS/sigma2)+2*p-n.
COMPUTE AIC=n*LN(RSS/n)+2*p.
COMPUTE SBc=n*LN(RSS/n)+p*LN(n).
COMPUTE Fvalue=(rsquare/(p-1))/((1-rsquare)/(n-p)).
COMPUTE Fsig=1-FCDF(Fvalue,(p-1),(n-p)).
* Reports for full model *.
PRINT /TITLE='************ STATISTICS FOR FULL MODEL ************'.
PRINT {n,p-1}
 /FORMAT='F8.0'
 /CLABELS='N','K'
 /TITLE='Sample size & number of predictors'.
PRINT {b,SEb,tvalue,tsig}
 /FORMAT='F8.3'
 /RNAMES=vnames
 /CLABELS='b','SE(b)','t value','2-tail P'
 /TITLE='Statistics for unstandardized coefficients (full model)'.
PRINT {rsquare,psquare,SQRT(sigma2)}
 /FORMAT='F8.3'
 /CLABEL='R-Square','P-Square','Res(SD)'
 /TITLE='Sample R-square, Adjusted (Population) Rho-square & Residual SD'.
PRINT {Fvalue,Fsig}
 /FORMAT='F8.4'
 /CLABEL='F value','Sig.'
 /TITLE='F test for R-square'.
*********** COMPUTING INFO FOR EVERY OTHER POSSIBLE MODEL ***********.
* Eliminate column of "1" from x array (will be added later inside a loop) *.
COMPUTE subx=x(:,(2:p)).
RELEASE x.
* Read the restricted list of subsets *.
GET subset /VAR=ALL /FILE='C:\Temp\SubsetsList.sav' /NAMES=xnames.
COMPUTE nvars=RSUM(subset).
* Initialize empty matrix where statistics for every model will be stored *.
COMPUTE totals=NROW(subset). 
COMPUTE allstats=MAKE(totals,6,0).
* Statistics for all possible models *.
* Info for full model is stored at first position (subsets were sorted in descending order) *.
COMPUTE allstats(1,1)=rsquare.
COMPUTE allstats(1,2)=psquare.
COMPUTE allstats(1,3)=SQRT(sigma2).
COMPUTE allstats(1,4)=MallowCp.
COMPUTE allstats(1,5)=AIc.
COMPUTE allstats(1,6)=SBc.
LOOP i=2 TO totals.  /* This LOOP needs that MXLOOPS is GE than "totals" *.
* Info for every other model *.
* Create x matrix at every step according to "i" subset pattern *.
. COMPUTE x={MAKE(n,1,1)}. /* Constant is added first *.
. LOOP j=1 TO NCOL(subx).
.  DO IF subset(i,j) EQ 1. /* Add variable only if its value in subset is 1 *.
.   COMPUTE x={x,subx(:,j)}.
.  END IF.
. END LOOP.
* Compute statistics for restricted model "i" *.
. COMPUTE p=NCOL(x).
. COMPUTE b=GINV(x)*y.
. COMPUTE ESS=T(b)*T(x)*y-n*meany&**2.
. COMPUTE RSS=TSS-ESS.
* Store important statistics in 'allstats' for later *.
. COMPUTE allstats(i,1)=ESS/TSS.                   /* R-square     *.
. COMPUTE allstats(i,2)=1-(1-ESS/TSS)*(n-1)/(n-p). /* Rho-square   *.
. COMPUTE allstats(i,3)=SQRT(RSS/(n-p)).           /* ResSD        *.
. COMPUTE allstats(i,4)=(RSS/sigma2)+2*p-n.        /* Mallow's Cp  *.
. COMPUTE allstats(i,5)=n*LN(RSS/n)+2*p.           /* Akaike IC    *.
. COMPUTE allstats(i,6)=n*LN(RSS/n)+p*LN(n).       /* Schwartz BC  *.
END LOOP.
* Report of number of models *.
PRINT {totals}
 /FORMAT='F8'
 /TITLE='Total number of models evaluated and exported to AllModels.sav'.
* Export ALL models to another temporary file for further analyses *.
COMPUTE export={subset,allstats}.
COMPUTE xnames={xnames,'R2','Rho2','ResSD','Cp','AIC','SBC'}.
SAVE export /OUTFILE='C:\Temp\AllModels.sav'/NAMES=xnames.
END MATRIX.

RESTORE.

* Read file with all models and start working *.
GET FILE='C:\Temp\AllModels.sav'.
COMPUTE nvars=SUM(!2).
COUNT nx = !2 (0) !2 (1)  .
PRESERVE.
SET ERRORS=   NONE
   /RESULTS=  NONE
   /PRINTBACK=NONE
   /MESSAGES= NONE.
RANK  VARIABLES=Cp (A) BY nvars /RANK /PRINT=NO.
RESTORE.
COMPUTE Pprime=nvars+1.
COMPUTE Psec=2*nvars-nx.
FORMAT Pprime Psec(F8) R2 TO SBC (F8.1).
VAR LABEL Pprime '.' /R2 'R' /Rho2 'Adj.R'.
****** GRAPHS FOR BEST MODELS ONLY ***.
* Graph Cp vs Pprime *.
TEMPORARY.
SELECT IF (RCp EQ 1).
GRAPH /SCATTERPLOT(OVERLAY)=pprime WITH Cp pprime psec.
* Graph Adjusted/Unadjusted R-square vs Pprime *.
TEMPORARY.
SELECT IF (RCp EQ 1).
GRAPH /SCATTERPLOT(OVERLAY)=pprime WITH R2 Rho2.
* Graph AIC/SCB vs Pprime *.
TEMPORARY.
SELECT IF (RCp EQ 1).
GRAPH /SCATTERPLOT(OVERLAY)=pprime WITH AIC SBC.
* Listing 5 best models for every number of predictors (thanks to Simon Freidin for his help) *.
FORMAT !2 nvars (F3) R2 TO SBC (F5.3).
VAL LABEL !2 0' ' 1'X'.
SORT CASES BY nvars(A) Cp(A) .
SPLIT FILE LAYERED BY nvars .
OMS /SELECT TABLES
 /IF COMMANDS='Summarize' SUBTYPES='Case Processing Summary'
 /DESTINATION VIEWER=NO.
TEMPORARY.
SELECT IF (RCp LE 5).
SUMMARIZE
  /TABLES=!2 R2 TO sbc
  /FORMAT=LIST NOCASENUM NOTOTAL
  /TITLE='Best subsets models'
  /MISSING=VARIABLE
  /CELLS=NONE.
OMSEND.
SPLIT FILE OFF.
SAVE OUTFILE='C:\Temp\AllModels.sav'.
GET FILE='C:\Temp\OriginalFile.sav'.

!ENDDEFINE.