PRESERVE.
SET PRINTBACK=ON.
*********************************************
* Methods (1 to 10):                        *
*********************************************
*  1: OR, Peto method                       *
*  2: OR, Mantel-Haenszel method            *
*  3: OR, Inverse variance (Woolf's) method *
*  4: RR, Mantel-Haenszel method            *
*  5: RR, Inverse variance (Woolf's) method *
*  6: RD, Mantel-Haenszel method            *
*  7: RD, Inverse variance (Woolf's) method *
*  8: OR, DerSimonian-Laird method          *
*  9: RR, DerSimonian-Laird method          *
* 10: RD, DerSimonian-Laird method          *
*********************************************
* Data: 4 variables with the number of      *
* events & total sample sizes in treated &  *
* control groups (order is: Xt Nt Xc Nc)    *
*********************************************
* (C) Marta Garca-Granero 08/2006          *
* send questions to: biostatistics@terra.es *
*********************************************.
RESTORE.

DEFINE METABIN(data=!TOKENS(4)/method=!TOKENS(1)).

* Adding extra variables to dataset *.
COMPUTE @xxxx=$SYSMIS. /* Flags the beginning of added variables *.
DO REPEAT A=evex totex evnex totnex /B=!data.
. COMPUTE A=B.
END REPEAT.
* Data modification by pseudo count method *.
COUNT nzeros1 = evex evnex (0).
COMPUTE #bi= totex-evex.
COMPUTE #di=totnex-evnex.
COUNT nzeros2 = #bi #di (0).
DO IF ((nzeros1 EQ 1) or (nzeros2 EQ 1)).
. COMPUTE evex=  evex+1/2.
. COMPUTE evnex= evnex+1/2.
. COMPUTE totex= totex+1.
. COMPUTE totnex=totnex+1.
END IF.
CACHE.
EXECUTE.
PRESERVE.
. SET ERRORS=NONE RESULTS=NONE.
. RANK trial /n into N /PRINT=NO.
RESTORE.

* Exclusion of undefined data trials *.
TEMPORARY.
SELECT IF (nzeros1 NE 2 and nzeros2 NE 2).

* MATRIX code *.
MATRIX.
PRINT  /TITLE="**** META-ANALYSIS OF BINARY RAW DATA ****".
COMPUTE randflag=0.
COMPUTE flag=0.
DO IF !method EQ 1.
. PRINT /TITLE=' FIXED EFFECT MODEL: ODDS-RATIO (PETO)'.
ELSE IF !method EQ 2.
. PRINT /TITLE=' FIXED EFFECT MODEL: ODDS-RATIO (MANTEL-HAENSZEL)'.
. COMPUTE flag=1.
ELSE IF !method EQ 3.
. PRINT /TITLE=' FIXED EFFECT MODEL: ODDS-RATIO (INVERSE VARIANCE . WOOLF)'.
. COMPUTE flag=1.
ELSE IF !method EQ 4.
. PRINT /TITLE=' FIXED EFFECT MODEL: RISK RATIO (MANTEL-HAENSZEL)'.
. COMPUTE flag=2.
ELSE IF !method EQ 5.
. PRINT /TITLE=' FIXED EFFECT MODEL: RISK RATIO (INVERSE VARIANCE . WOOLF)'.
. COMPUTE flag=2.
ELSE IF !method EQ 6.
. PRINT /TITLE=' FIXED EFFECT MODEL: RISK DIFFERENCE (MANTEL-HAENSZEL)'.
. COMPUTE flag=3.
ELSE IF !method EQ 7.
. PRINT /TITLE=' FIXED EFFECT MODEL: RISK DIFFERENCE (INVERSE VARIANCE . WOOLF)'.
. COMPUTE flag=3.
ELSE IF !method EQ 8.
. PRINT /TITLE=' RANDOM EFFECTS MODEL: ODDS-RATIO (DERSIMONIAN-LAIRD)'.
. COMPUTE flag=1.
. COMPUTE randflag=1.
ELSE IF !method EQ 9.
. PRINT /TITLE=" RANDOM EFFECTS MODEL: RISK RATIO (DERSIMONIAN-LAIRD)".
. COMPUTE flag=2.
. COMPUTE randflag=1.
ELSE.
. PRINT /TITLE=" RANDOM EFFECTS MODEL: RISK DIFFERENCE (DERSIMONIAN-LAIRD)".
. COMPUTE flag=3.
. COMPUTE randflag=1.
END IF.
* Input data, basic calculations & input data report *.
GET numtr /VAR=n.
GET id    /VAR=trial.
COMPUTE k=NROW(id).
GET trial /VAR=study.
GET ai    /VAR=evex.
GET ci    /VAR=evnex.
GET n1i   /VAR=totex.
GET n0i   /VAR=totnex.
COMPUTE bi=n1i-ai.
COMPUTE di=n0i-ci.
COMPUTE ni=n1i+n0i.
COMPUTE sumai=MSUM(ai).
COMPUTE sumbi=MSUM(bi).
COMPUTE sumci=MSUM(ci).
COMPUTE sumdi=MSUM(di).
COMPUTE sumn1i=MSUM(n1i).
COMPUTE sumn0i=MSUM(n0i).
COMPUTE m1i=ai+ci.
COMPUTE m0i=bi+di.
DO IF (flag EQ 0) OR (flag EQ 1).
. PRINT {ai,bi,ci,di;sumai,sumbi,sumci,sumdi}
 /FORMAT='F8.1'
 /RNAMES=trial
 /CLABELS='CaExp+','CoExp+','CaExp-','CoExp-'
 /TITLE='INPUT DATA (pseudo-count modified if necessary)'.
ELSE IF (flag EQ 2) OR (flag EQ 3).
. PRINT {ai,n1i,ci,n0i;sumai,sumn1i,sumci,sumn0i}
 /FORMAT='F8.1'
 /RNAMES=trial
 /CLABELS='Ev+EXP+','TotEXP+','Ev+EXP-','TotEXP-'
 /TITLE='INPUT DATA (pseudo-count modified if necessary)'.
END IF.
PRINT k
 /FORMAT='F8.0'
 /TITLE="Number of trials analysed (K)".
********************* INDIVIDUAL OR/RR/RD *********************.
COMPUTE colname={'OR','SE(LnOR)','Lower','Upper'}.
DO IF (flag EQ 0).
* Peto individual OR *.
. COMPUTE eai=n1i&*m1i&/ni.
. COMPUTE wi=n1i&*n0i&*m1i&*m0i&/((ni&**3)-ni).
. COMPUTE measure=EXP((ai-eai)/wi).
. COMPUTE semeasur=SQRT(1/wi).
. COMPUTE percwi=100&*wi/MSUM(wi).
. COMPUTE cilow=EXP(LN(measure)-1.96&*semeasur).
. COMPUTE ciup= EXP(LN(measure)+1.96&*semeasur).
ELSE IF (flag EQ 1).
* Individual OR (IV, MH & DL)*.
. COMPUTE measure=(ai&*di)&/(ci&*bi).
. COMPUTE semeasur=SQRT((1/ai)+(1/bi)+(1/ci)+(1/di)).
. DO IF (!method EQ 2).
.  COMPUTE wi=bi&*ci/ni. /* MH wi *.
. ELSE.
.  COMPUTE wi=(1/semeasur)&**2.
. END IF.
. COMPUTE percwi=100*wi/MSUM(wi).
. COMPUTE cilow=EXP(LN(measure)-1.96&*semeasur).
. COMPUTE ciup= EXP(LN(measure)+1.96&*semeasur).
ELSE IF (flag EQ 2).
* Individual RR (IV, MH & DL)*.
. COMPUTE colname={'RR','SE(LnRR)','Lower','Upper'}.
. COMPUTE measure=(ai&/n1i)&/(ci&/n0i).
. COMPUTE semeasur=SQRT((1/ai)-(1/n1i)+(1/ci)-(1/n0i)).
. DO IF (!method EQ 4).
.  COMPUTE wi=ci&*(ai+bi)/ni. /* MH wi *.
. ELSE.
.  COMPUTE wi=(1/semeasur)&**2.
. END IF.
. COMPUTE percwi=100&*wi/MSUM(wi).
. COMPUTE cilow=EXP(ln(measure)-1.96&*semeasur).
. COMPUTE ciup= EXP(ln(measure)+1.96&*semeasur).
ELSE IF (flag EQ 3).
* Individual RD *.
. COMPUTE colname={'RD','SE(RD)','Lower','Upper'}.
. COMPUTE measure=(ai&/n1i)-(ci&/n0i).
. COMPUTE semeasur=SQRT(ai&*bi&/(n1i&**3)+ci&*di&/(n0i&**3)).
. DO IF (!method EQ 6).
.  COMPUTE wi=n1i&*n0i/ni. /* MH wi *.
. ELSE.
.  COMPUTE wi=(1/semeasur)&**2.
. END IF.
. COMPUTE percwi=100&*wi/MSUM(wi).
. COMPUTE cilow=measure-1.96&*semeasur.
. COMPUTE ciup= measure+1.96&*semeasur.
END IF.
********************* SUMMARY OR/RR/RD (ALL FIXED) *********************.
DO IF (!method EQ 2).
* Mantel-Haenszel summary OR *.
. COMPUTE summary=MSUM(wi&*measure)/MSUM(wi).
. COMPUTE pi=(ai+di)&/ni.
. COMPUTE qi=(bi+ci)&/ni.
. COMPUTE ri=(ai&*di)&/ni.
. COMPUTE si=(bi&*ci)&/ni.
. COMPUTE num1=MSUM(pi&*ri).
. COMPUTE num2=MSUM(pi&*si+qi&*ri).
. COMPUTE num3=MSUM(qi&*si).
. COMPUTE den1=MSUM(ri).
. COMPUTE den2=MSUM(si).
. COMPUTE sesummar=SQRT((num1/(2*(den1**2)))+(num2/(2*den1*den2))+(num3/(2*(den2**2)))).
. COMPUTE cilowsm=summary*EXP(-1.96*sesummar).
. COMPUTE ciupwsm=summary*EXP( 1.96*sesummar).
. RELEASE pi,qi,ri,si,num1,num2,num3,den1,den2.
ELSE IF (!method EQ 1) OR (!method EQ 3) OR (!method EQ 8).
* Peto/IV/DL summary OR *.
. COMPUTE num=MSUM(wi&*LN(measure)).
. COMPUTE den=MSUM(wi).
. COMPUTE summary=EXP(num/den).
. COMPUTE sesummar=1/SQRT(den).
. COMPUTE cilowsm=EXP((num/den)-1.96*sesummar).
. COMPUTE ciupwsm=EXP((num/den)+1.96*sesummar).
. RELEASE num,den.
ELSE IF (!method EQ 4).
* Mantel-Haenszel RR *.
. COMPUTE summary=MSUM(wi&*measure)/MSUM(wi).
. COMPUTE num1=MSUM((m1i&*n1i&*n0i-ai&*ci&*ni)&/(ni&**2)).
. COMPUTE den1=MSUM(ai&*n0i&/ni).
. COMPUTE den2=MSUM(ci&*n1i&/ni).
. COMPUTE sesummar=SQRT(num1/(den1*den2)).
. COMPUTE cilowsm=summary*exp(-1.96*sesummar).
. COMPUTE ciupwsm=summary*exp( 1.96*sesummar).
. RELEASE num1,den1,den2.
ELSE IF (!method EQ 5) OR (!method EQ 9).
* IV (Woolf)/DL RR *.
. COMPUTE num=MSUM(wi&*ln(measure)).
. COMPUTE den=MSUM(wi).
. COMPUTE summary=EXP(num/den).
. COMPUTE sesummar=1/SQRT(den).
. COMPUTE cilowsm=EXP((num/den)-1.96*sesummar).
. COMPUTE ciupwsm=EXP((num/den)+1.96*sesummar).
. RELEASE num,den.
ELSE IF (!method EQ 6).
* Mantel-Haenszel RD *.
. COMPUTE summary=MSUM(measure&*wi)/MSUM(wi).
. COMPUTE nnt=1/ABS(summary).
. COMPUTE p=MSUM((ai&*bi&*(n0i&**3)+ci&*di&*(n1i&**3))/(n1i&*n0i&*(ni&**2))).
. COMPUTE q=MSUM(n1i&*n0i/ni).
. COMPUTE sesummar=SQRT(p/(q&**2)).
. COMPUTE cilowsm=summary-1.96*sesummar.
. COMPUTE ciupwsm=summary+1.96*sesummar.
. RELEASE p,q.
ELSE IF (!method EQ 7) OR (!method EQ 10).
* IV (Woolf)/DL RD *.
. COMPUTE summary=MSUM(measure&*wi)/MSUM(wi).
. COMPUTE nnt=1/ABS(summary).
. COMPUTE sesummar=1/SQRT(MSUM(wi)).
. COMPUTE cilowsm=summary-1.96*sesummar.
. COMPUTE ciupwsm=summary+1.96*sesummar.
END IF.
********************* HETEROGENEITY *********************.
DO IF (flag NE 3). /* Log scale *.
. DO IF (!method EQ 2) OR (!method EQ 4). /* MH exception *.
.  COMPUTE wpi=(1/semeasur)&**2.
.  COMPUTE het_chi=MSUM(wpi&*(LN(measure)-LN(summary))&**2).
. ELSE.
.  COMPUTE het_chi=MSUM(wi&*(LN(measure)-LN(summary))&**2).
. END IF.
ELSE. /* Linear scale *.
. DO IF (!method EQ 6).                   /* MH exception *.
.  COMPUTE wpi=(1/semeasur)&**2.
.  COMPUTE het_chi=MSUM(wpi&*(measure-summary)&**2).
. ELSE.
.  COMPUTE het_chi=MSUM(wi&*(measure-summary)&**2).
. END IF.
END IF.
COMPUTE het_sig=1-CHICDF(het_chi,k-1).
PRINT /TITLE='HETEROGENEITY TEST AND STATISTICS'.
PRINT {het_chi,het_sig}
 /FORMAT='F8.4'
 /CLABELS='Chi^2','Sig.'
 /TITLE='Cochran Q heterogeneity test (df=K-1) . H0: Homogeneity'.
DO IF het_chi GT (k-1). /* Computing I-square with 95%CI *.
. DO IF het_sig GT 0.10.
.  PRINT /TITLE='WARNING: Q p>0.10, but some heterogeneity exists!'.
. END IF.
. COMPUTE h=SQRT(het_chi/(k-1)).
. COMPUTE isqr=100*(het_chi-(k-1))/het_chi.
. DO IF het_chi GT k.
.  COMPUTE eeh=LN(h)/(SQRT(2*het_chi)-SQRT(2*k-3)).
. ELSE IF het_chi LE k.
.  COMPUTE eeh=SQRT((1-(1/(3*(k-2)**2)))/(2*(k-2))).
. END IF.
. COMPUTE lowh=h*EXP(-1.96*eeh).
. COMPUTE upph=h*EXP(1.96*eeh).
. COMPUTE lowisqr=100*(lowh**2-1)/(lowh**2).
. DO IF lowisqr LT 0.
.  COMPUTE lowisqr=0.
. END IF.
. COMPUTE uppisqr=100*(upph**2-1)/(upph**2).
. DO IF uppisqr GT 100.
.  COMPUTE uppisqr=100.
. END IF.
. PRINT {isqr,lowisqr,uppisqr}
 /FORMAT='F8.1'
 /CLABELS='I^2(%)','Low95 CI','Upp95 CI'
 /TITLE='Heterogeneity statistic: 25%(low), 50%(moderate), 75%(high)'.
END IF.
* Between-trials variance (tau-square) *.
COMPUTE tau=(het_chi-(k-1))/(MSUM(wi)-(MSUM(wi&**2))/MSUM(wi)).
DO IF tau GT 0.
. PRINT tau 
 /FORMAT='F8.3'
 /TITLE='Tau-square (between trials variance)'.
ELSE.
. COMPUTE tau=0.
END IF.
DO IF (tau EQ 0) AND (randflag EQ 1).
. PRINT /TITLE='Tau-square=0 (random. & fixed-effect models will yield'
+ ' identical results)'.
END IF.
********************* SUMMARY OR/RR/RD (RANDOM) *********************
* For random effects models, modify wi using tau-square and compute again
  summary measures *.
DO IF (randflag EQ 1).
. PRINT /TITLE="INDIVIDUAL MEASURES & TAU-SQUARE MODIFIED WEIGHTS".
. COMPUTE vartilda=tau+semeasur&**2.
. COMPUTE wi=1/vartilda.
. COMPUTE percwi=100&*wi/MSUM(wi).
. DO IF (!method EQ 8).  /* DL summary OR *.
.  COMPUTE num=MSUM(wi&*LN(measure)).
.  COMPUTE den=MSUM(wi).
.  COMPUTE summary=EXP(num/den).
.  COMPUTE sesummar=1/SQRT(den).
.  COMPUTE cilowsm=EXP((num/den)-1.96*sesummar).
.  COMPUTE ciupwsm=EXP((num/den)+1.96*sesummar).
.  RELEASE num,den.
. ELSE IF (!method EQ 9). /* DL RR *.
.  COMPUTE num=MSUM(wi&*ln(measure)).
.  COMPUTE den=MSUM(wi).
.  COMPUTE summary=EXP(num/den).
.  COMPUTE sesummar=1/SQRT(den).
.  COMPUTE cilowsm=EXP((num/den)-1.96*sesummar).
.  COMPUTE ciupwsm=EXP((num/den)+1.96*sesummar).
.  RELEASE num,den.
. ELSE IF (!method EQ 10). /* DL RD *.
.  COMPUTE summary=MSUM(measure&*wi)/MSUM(wi).
.  COMPUTE nnt=1/ABS(summary).
.  COMPUTE sesummar=1/SQRT(MSUM(wi)).
.  COMPUTE cilowsm=summary-1.96*sesummar.
.  COMPUTE ciupwsm=summary+1.96*sesummar.
. END IF.
ELSE.
. PRINT /TITLE="INDIVIDUAL MEASURES & WEIGHTS".
END IF.
**************************** REPORTS ****************************.
COMPUTE colname={colname,'%Weight'}.
PRINT {measure,semeasur,cilow,ciup,percwi}
 /FORMAT='F8.3'
 /RNAMES=trial
 /CNAMES=colname
 /TITLE='                                  95% CI'.
DO IF (randflag EQ 1).
. PRINT /TITLE="DERSIMONIAN-LAIRD STATISTIC".
END IF.
PRINT {summary,sesummar,cilowsm,ciupwsm,100}
 /FORMAT='F8.3'
 /RLABELS=' Overall'
 /CNAMES=colname
 /TITLE='SUMMARY                           95% CI'.
DO IF (flag NE 3).
. COMPUTE a_chi=(LN(summary)/sesummar)**2.
ELSE.
. COMPUTE a_chi=(summary/sesummar)**2.
END IF.
COMPUTE a_sig=1-CHICDF(a_chi,1).
DO IF (flag EQ 3) AND (a_sig LE 0.05).
. PRINT nnt
 /FORMAT='F8.2'
 /TITLE='Number Needed to Treat (or Number Needed to Harm)'.
END IF.
PRINT {a_chi,SQRT(a_chi),a_sig}
 /FORMAT='F8.4'
 /CLABELS='Chi^2','Z','Sig.'
 /TITLE='Association Chi-square (& Z) statistic (df=1) . H0: No association'.
* Exporting data for report & forest-plot *.
COMPUTE data={id,measure,semeasur,cilow,ciup,percwi;numtr(1)+1,summary,sesummar,cilowsm,ciupwsm,100}.
COMPUTE namevec={'trial','measure','semeasur','cilow','ciup','percwi'}.
SAVE data /OUTFILE='C:\Temp\extracols.sav' /NAMES=namevec.
COMPUTE data={numtr(1)+1,sumai,sumn1i,sumci,sumn0i}.
COMPUTE namevec={'trial','evex','totex','evnex','totnex'}.
SAVE data /OUTFILE='C:\Temp\extrarow.sav' /NAMES=namevec.
END MATRIX.

* Adding extra data to current file *.
ADD FILES /FILE=*
 /FILE='C:\temp\extrarow.sav'.
MATCH FILES /FILE=*
 /FILE='C:\Temp\extracols.sav' /BY trial.
IF study='' study = 'Total' .
DO REPEAT A=!data /B=evex totex evnex totnex.
. IF MISSING(A) A=B.
END REPEAT.

* Report with correct label for measure (with thanks to Richard Ristow) *.
VAR LABEL cilow 'Lower 95%CI' /ciup 'Upper 95%CI' /measure 'OR' /percwi'Weights(%)'.
!IF ((!method !EQ 4)!OR(!method !EQ 5)!OR(!method !EQ 9)) !THEN
. VAR LABEL measure 'RR'.
!IFEND.
!IF ((!method !EQ 6)!OR(!method !EQ 7)!OR(!method !EQ 10)) !THEN
. VAR LABEL measure 'RD'.
!IFEND.
SUMMARIZE
 /TABLES=study !data measure cilow ciup percwi
 /FORMAT=VALIDLIST NOCASENUM TOTAL
 /TITLE='Input data & Summary measures'
 /MISSING=VARIABLE /CELLS=NONE.

*  Forest plot with individual and aggregated Summary measures (thanks again to RR)*.
!LET !title=!UNQUOTE('Fixed Effects Model').
!LET !template=!UNQUOTE('C:\Temp\Forest1.sgt').
!IF ((!method !EQ 6)!OR(!method !EQ 7)) !THEN
. !LET !template=!UNQUOTE('C:\Temp\Forest2.sgt').
!IFEND.
!IF ((!method !EQ 8)!OR(!method !EQ 9)) !THEN
. !LET !title=!UNQUOTE('Random Effects Model').
!IFEND.
!IF (!method !EQ 10) !THEN
. !LET !title=!UNQUOTE('Random Effects Model').
. !LET !template=!UNQUOTE('C:\Temp\Forest2.sgt').
!IFEND.
DO IF $casenum EQ 1. 
. WRITE OUTFILE 'C:\Temp\ForestPlot.sps'
  / "GRAPH /HILO(SIMPLE)=VALUE( ciup cilow measure ) BY study"
  / " /TITLE="!QUOTE(!QUOTE(!title))" /TEMPLATE="!QUOTE(!QUOTE(!template))".".
END IF.
SORT CASES BY trial(D).
INCLUDE 'C:\Temp\ForestPlot.sps'.

* Reverting dataset to original status *.
SORT CASES BY trial(A).
SELECT IF study NE 'Total'.
COMPUTE @yyyy=$SYSMIS. /* Flags the last added variable *.
MATCH FILES /FILE=* /DROP= @xxxx TO @yyyy.

* End of syntax *.
!ENDDEFINE.

* Accuracy of the MACRO: the output was compared to table 15.4 ( see below)
* of 'Systematic Reviews in Health Care' (p 303-306), using the same dataset.

*                            Table 15.4
*  Method         Estimate (95% CI)        Sig of effect     Heterogeneity
*  ---------------------------------------------------------------------------
*  Odds Ratio:
*    Peto         0.59 (0.51 to 0.69)      z=7.05 (p<0.001)  X=38.5 (p<0.001)
*    M-H          0.59 (0.51 to 0.69)      z=6.98 (p<0.001)  X=38.9 (p<0.001)
*    Inv Var      0.60 (0.52 to 0.70)      z=6.70 (p<0.001)  X=38.8 (p<0.001)
*    DerS-Laird   0.54 (0.34 to 0.85)      z=2.64 (p=0.008)

*  Risk Ratio:
*    M-H          0.79 (0.74 to 0.85)      z=6.95 (p<0.001)  X=29.8 (p<0.001)
*    Inv Var      0.80 (0.75 to 0.85)      z=7.14 (p<0.001)  X=29.7 (p<0.001)
*    DerS-Laird   0.77 (0.64 to 0.92)      z=2.93 (p=0.003)

*  Risk Diff:
*    M-H         -0.117 (-0.149 to -0.085) z=7.13 (p<0.001)  X=33.1 (p<0.001)
*    Inv Var     -0.127 (-0.158 to -0.095) z=7.86 (p<0.001)  X=32.7 (p<0.001)
*    DerS-Laird  -0.124 (-0.211 to -0.038) z=2.81 (p=0.005)
*  --------------------------------------------------------------------------. *.

* Dataset for Table 15.4  *.
DATA LIST LIST/trial(F4.0) year(A5) study(A10) xt nt xc nc(4 F8.0).
BEGIN DATA
1	1992 	Brart-Fr 	 55	133	 62	131
2	1992 	Brart-Bg 	281	656	319	664
3	1997 	Gagnon    	139	209	142	204
4	1989 	Hodnett   	 30	 72	 43	 73
5	1991 	Kennell   	 24	212	 55	200
6	1998 	Langer    	205	361	303	363
END DATA.
VAR LABEL trial 'Trial ID number' /year 'Publication year' /study 'Trial ID name'
 /xt 'Events (epidurals) in caregiver group' /nt 'Total N caregiver group'
 /xc 'Events (epidurals) in standard care group' /nc 'Total N standard care group'.

DOCUMENT 'Rates of use of epidural anaesthesia in trials of caregiver support
          (from p 303 of Systematic Reviews in Health Care, BMJ Books)'.

* MACRO calls (DON'T use the keyword "TO" to list the 4 variables) *.
METABIN data = xt nt xc nc method = 1.
METABIN data = xt nt xc nc method = 2.
METABIN data = xt nt xc nc method = 3.
METABIN data = xt nt xc nc method = 4.
METABIN data = xt nt xc nc method = 5.
METABIN data = xt nt xc nc method = 6.
METABIN data = xt nt xc nc method = 7.
METABIN data = xt nt xc nc method = 8.
METABIN data = xt nt xc nc method = 9.
METABIN data = xt nt xc nc method =10.
