PRESERVE.
SET PRINTBACK=ON.
******************************************************
* Methods (1 to 8):                                  *
******************************************************
* 1: WMD, Inverse Variance (Woolf's) method          *
* 2: Hedges G, Inverse Variance (Woolf's) method     *
* 3: Cohen's D, Inverse variance (Woolf's) method    *
* 4: Glass' Delta, Inverse variance (Woolf's) method *
* 5: WMD, DerSimonian-Laird method                   *
* 6: Hedges G, DerSimonian-Laird method              *
* 7: Cohen's D, DerSimonian-Laird method             *
* 8: Glass' Delta, DerSimonian-Laird method          *
******************************************************
* Data: 6 variables with mean, sd & sample sizes     *
* in treated & control groups (order as indicated)   *
******************************************************
* (C) Marta Garca-Granero 08/2006                   *
* send questions to: biostatistics@terra.es          *
******************************************************.

RESTORE.

DEFINE METACONT(method=!TOKENS(1) /data=!TOKENS(6)).
* Adding variables *.
COMPUTE @xxxx=$SYSMIS. /* Flags the beginning of added variables *.
DO REPEAT A=@meantr @sdtr @ntr @meanct @sdct @nct /B=!data.
- COMPUTE A=B.
END REPEAT.
CACHE.
EXECUTE.
* MATRIX code *.
MATRIX.
PRINT  /TITLE="**** META-ANALYSIS OF CONTINUOUS DATA ****".
COMPUTE randflag=0. /* Activate if method is random *.
COMPUTE flag=0.     /* Activate if non-standardised MD are used *.
DO IF !method EQ 1.
- PRINT /TITLE=" FIXED EFFECT MODEL: WEIGHTED MEAN DIFFERENCES (INVERSE VARIANCE)".
- COMPUTE flag=1.
ELSE IF !method EQ 2.
- PRINT /TITLE=" FIXED EFFECT MODEL: HEDGES' G (INVERSE VARIANCE)".
ELSE IF !method EQ 3.
- PRINT /TITLE=" FIXED EFFECT MODEL: COHEN'S D (INVERSE VARIANCE)".
ELSE IF !method EQ 4.
- PRINT /TITLE=" FIXED EFFECT MODEL: GLASS' DELTA (INVERSE VARIANCE)".
ELSE IF !method EQ 5.
- PRINT /TITLE=" RANDOM EFFECTS MODEL: WEIGHTED MEAN DIFFERENCES (DERSIMONIAN-LAIRD)".
- COMPUTE randflag=1.
- COMPUTE flag=1.
ELSE IF !method EQ 6.
- PRINT /TITLE=" RANDOM EFFECTS MODEL: HEDGES' G (DERSIMONIAN-LAIRD)".
- COMPUTE randflag=1.
ELSE IF !method EQ 7.
- PRINT /TITLE=" RANDOM EFFECTS MODEL: COHEN'S D (DERSIMONIAN-LAIRD)".
- COMPUTE randflag=1.
ELSE IF !method EQ 8.
- PRINT /TITLE=" RANDOM EFFECTS MODEL: GLASS' DELTA (DERSIMONIAN-LAIRD)".
- COMPUTE randflag=1.
END IF.
* Input data *.
GET trial /VAR=study.
GET mean1 /VAR=@meantr.
GET sd1   /VAR=@sdtr.
GET n1    /VAR=@ntr.
GET mean2 /VAR=@meanct.
GET sd2   /VAR=@sdct.
GET n2    /VAR=@nct.
* General calculations & MD report *.
COMPUTE n=n1+n2.
COMPUTE k=NROW(n).
COMPUTE totaln=MSUM(n).
PRINT k
 /FORMAT='F8.0'
 /TITLE="Number of trials (K)".
PRINT totaln
 /FORMAT='F8.0'
 /TITLE="Total sample size (N)".
********************* INDIVIDUAL WMD/G/D/Delta *********************.
DO IF (!method EQ 1) OR (!method EQ 5). /* WMD *.
- COMPUTE colname={'MeanDiff','SE(diff)','Lower','Upper','%Weight'}.
- COMPUTE measure=mean1-mean2.
- COMPUTE vard=(sd1&**2/n1)+(sd2&**2/n2).
- COMPUTE semeasur=SQRT(vard).
- COMPUTE cilow=measure-1.96*semeasur.
- COMPUTE ciup= measure+1.96*semeasur.
ELSE IF (!method EQ 2) OR (!method EQ 6). /* Hedges G *.
- COMPUTE colname={'Hedges G','SE(G)','Lower','Upper','%Weight'}.
- COMPUTE poolvar=((n1-1)&*(sd1&**2)+(n2-1)&*(sd2&**2))/(n-2) .
- COMPUTE d=(mean1-mean2)/SQRT(poolvar).
- COMPUTE measure=d&*(1-(3/((4&*n)-9))).
- COMPUTE vard=(n/(n1&*n2))+(measure&**2/(2*(n-3.94))).
- COMPUTE semeasur=SQRT(vard).
- COMPUTE cilow=measure-1.96*semeasur.
- COMPUTE ciup= measure+1.96*semeasur.
ELSE IF (!method EQ 3) OR (!method EQ 7). /* Cohen D *.
- COMPUTE colname={'Cohen D','SE(D)','Lower','Upper','%Weight'}.
- COMPUTE poolvar=((n1-1)&*(sd1&**2)+(n2-1)&*(sd2&**2))/(n-2) .
- COMPUTE measure=(mean1-mean2)/SQRT(poolvar).
- COMPUTE vard=(n/(n1&*n2))+(measure&**2/(2*(n-2))).
- COMPUTE semeasur=SQRT(vard).
- COMPUTE cilow=measure-1.96*semeasur.
- COMPUTE ciup= measure+1.96*semeasur.
ELSE IF (!method EQ 4) OR (!method EQ 8). /* Glass Delta *.
- COMPUTE colname={'Glass D','SE(GD)','Lower','Upper','%Weight'}.
- COMPUTE measure=(mean1-mean2)/sd2.
- COMPUTE vard=(n/(n1&*n2))+(measure&**2/(2*(n2-1))).
- COMPUTE semeasur=SQRT(vard).
- COMPUTE cilow=measure-1.96*semeasur.
- COMPUTE ciup= measure+1.96*semeasur.
END IF.
********** SUMMARY WMD/G/D/Delta (ALL FIXED) ************.
COMPUTE wi=1/vard.
COMPUTE percwi=100*wi/MSUM(wi).
COMPUTE summary=MSUM(measure&*wi)/MSUM(wi).
COMPUTE sesummar=1/SQRT(MSUM(wi)).
COMPUTE cilowsm=summary-1.96*sesummar.
COMPUTE ciupwsm=summary+1.96*sesummar.
********************* HETEROGENEITY *********************.
COMPUTE het_chi=MSUM(wi&*(measure-summary)&**2).
COMPUTE het_sig=1-CHICDF(het_chi,k-1).
PRINT /TITLE='HETEROGENEITY TEST AND STATISTICS'.
PRINT {het_chi,het_sig}
 /FORMAT='F8.4'
 /CLABELS='Chi^2','Sig.'
 /TITLE='Cochran Q heterogeneity test (df=K-1) - H0: Homogeneity'.
DO IF het_chi GT (k-1). /* Computing I-square with 95%CI *.
- DO IF het_sig GT 0.10.
-  PRINT /TITLE='WARNING: Q p>0.10, but some heterogeneity exists!'.
- END IF.
- COMPUTE h=SQRT(het_chi/(k-1)).
- COMPUTE isqr=100*(het_chi-(k-1))/het_chi.
- DO IF het_chi GT k.
-  COMPUTE eeh=LN(h)/(SQRT(2*het_chi)-SQRT(2*k-3)).
- ELSE IF het_chi LE k.
-  COMPUTE eeh=SQRT((1-(1/(3*(k-2)**2)))/(2*(k-2))).
- END IF.
- COMPUTE lowh=h*EXP(-1.96*eeh).
- COMPUTE upph=h*EXP(1.96*eeh).
- COMPUTE lowisqr=100*(lowh**2-1)/(lowh**2).
- DO IF lowisqr LT 0.
-  COMPUTE lowisqr=0.
- END IF.
- COMPUTE uppisqr=100*(upph**2-1)/(upph**2).
- DO IF uppisqr GT 100.
-  COMPUTE uppisqr=100.
- END IF.
- PRINT {isqr,lowisqr,uppisqr}
 /FORMAT='F8.1'
 /CLABELS='I^2(%)','Low95 CI','Upp95 CI'
 /TITLE='Heterogeneity statistic: 25%(low), 50%(moderate), 75%(high)'.
END IF.
* Between-trials variance (tau-square) *.
COMPUTE tau=(het_chi-(k-1))/(MSUM(wi)-(MSUM(wi&**2))/MSUM(wi)).
DO IF tau GT 0.
- PRINT tau 
 /FORMAT='F8.3'
 /TITLE='Tau-square (between trials variance)'.
ELSE.
- COMPUTE tau=0.
END IF.
DO IF (tau EQ 0) AND (randflag EQ 1).
- PRINT /TITLE='Tau-square=0 (random- & fixed-effect models will yield'
  +' identical results)'.
END IF.
********************* SUMMARY WMD/G/D/Delta (RANDOM) *********************
* For random effects models only, modify wi using tau-square and compute
  again summary measures (after measuring heterogeneity) *.
DO IF (randflag EQ 1).
- COMPUTE wi=1/(tau+semeasur&**2).
- COMPUTE percwi=100&*wi/MSUM(wi).
- COMPUTE summary=MSUM(measure&*wi)/MSUM(wi).
- COMPUTE sesummar=1/SQRT(MSUM(wi)).
- COMPUTE cilowsm=summary-1.96*sesummar.
- COMPUTE ciupwsm=summary+1.96*sesummar.
- PRINT /TITLE='INDIVIDUAL MEASURES & TAU-SQUARE MODIFIED WEIGHTS'.
ELSE.
- PRINT /TITLE='INDIVIDUAL MEASURES & WEIGHTS'.
END IF.
* Significance of summary measure *.
COMPUTE a_chi=(summary/sesummar)&**2.
COMPUTE a_sig=1-CHICDF(a_chi,1).
**************************** REPORTS ****************************.
PRINT {measure,semeasur,cilow,ciup,percwi}
 /FORMAT='F8.3'
 /RNAMES=trial
 /CNAMES=colname
 /TITLE='                                  95% CI'.
DO IF (randflag EQ 1).
- PRINT /TITLE='DERSIMONIAN-LAIRD STATISTIC'.
END IF.
PRINT {summary,sesummar,cilowsm,ciupwsm,100}
 /FORMAT='F8.3'
 /RLABELS=' Overall'
 /CNAMES=colname
 /TITLE='SUMMARY                           95% CI'.
* Extra statistics for standardised mean differences *.
DO IF (flag NE 1). /* Don't compute for WMD *.
- COMPUTE r=ABS(summary&/SQRT(summary&**2+(4&*(totaln-2)&/totaln))).
- COMPUTE besd_l=0.5-r/2.
- COMPUTE besd_u=0.5+r/2.
- PRINT {r,besd_l,besd_u}
  /FORMAT='F8.3'
  /TITLE='Point biserial R & BESD'
  /CLABELS='R','BESD(L)','BESD(U)'.
- DO IF ABS(summary GT 0.2). /* Compute only if G/D/Delta are GT 0.2 *.
-  COMPUTE ofsn=k*(ABS(summary/0.2)-1).
-  PRINT ofsn
  /FORMAT='F8.0'
  /TITLE="Orwin's Fail Safe N for small effect size (0.2)".
- END IF.
END IF.
PRINT {a_chi,SQRT(a_chi),a_sig}
 /FORMAT='F8.4'
 /CLABELS='Chi^2','Z','Sig.'
 /TITLE='Association Chi-square (& Z) statistic (df=1) - H0: No association'.
**************************** EXPORTING DATA ****************************.
COMPUTE data={measure,semeasur,cilow,ciup,percwi;summary,sesummar,cilowsm,ciupwsm,100}.
COMPUTE namevec={'measure','semeasur','cilow','ciup','percwi'}.
SAVE data /OUTFILE='C:\Temp\extracols.sav' /NAMES=namevec.
COMPUTE data={MSUM(n1),MSUM(n2)}.
COMPUTE namevec={'@ntr','@nct'}.
SAVE data /OUTFILE='C:\Temp\extrarow.sav' /NAMES=namevec.
END MATRIX.
* Adding extra data to current file *.
ADD FILES /FILE=*
 /FILE='C:\temp\extrarow.sav'.
MATCH FILES /FILE=*
 /FILE='C:\Temp\extracols.sav'.
IF study='' study = 'Total' .
IF (MISSING(trial)) trial = $casenum .
DO REPEAT A=!data /B=@meantr @sdtr @ntr @meanct @sdct @nct .
- IF MISSING(A) A=B.
END REPEAT.
* Report with correct label for measure (with thanks to Richard Ristow) *.
VAR LABEL cilow 'Lower 95%CI' /ciup 'Upper 95%CI' /measure 'WMD' /percwi 'Weights(%)'.
!IF ((!method !EQ 2)!OR(!method !EQ 4)) !THEN
- VAR LABEL measure 'Hedges G'.
!IFEND.
!IF ((!method !EQ 3)!OR(!method !EQ 7)) !THEN
- VAR LABEL measure 'Cohen D'.
!IFEND.
!IF ((!method !EQ 4)!OR(!method !EQ 8)) !THEN
- VAR LABEL measure 'Glass Delta'.
!IFEND.
SUMMARIZE
 /TABLES=study !data measure cilow ciup percwi
 /FORMAT=VALIDLIST NOCASENUM TOTAL
 /TITLE='Input data & Summary measures'
 /MISSING=VARIABLE /CELLS=NONE.
*  Forest plot with individual and aggregated Summary measures (thanks again to RR)*.
!IF (!method !LE 4) !THEN
- !LET !title=!UNQUOTE('Fixed Effects Model').
!ELSE
- !LET !title=!UNQUOTE('Random Effects Model').
!IFEND.
DO IF $casenum EQ 1. 
- WRITE OUTFILE 'C:\Temp\ForestPlot.sps'
  / "GRAPH /HILO(SIMPLE)=VALUE( ciup cilow measure ) BY study"
  / " /TITLE="!QUOTE(!QUOTE(!title))" /TEMPLATE='C:\Temp\Forest2.sgt'.".
END IF.
SORT CASES BY trial(D).
INCLUDE 'C:\Temp\ForestPlot.sps'.
* Reverting dataset to original status *.
SORT CASES BY trial(A).
SELECT IF study NE 'Total'.
COMPUTE @yyyy=$SYSMIS. /* Flags the last added variable *.
MATCH FILES /FILE=* /DROP= @xxxx TO @yyyy.
* End of syntax *.
!ENDDEFINE.

* Accuracy of the MACRO: the output for 6 of 8 MACRO calls was compared to 
* table 15.6 ( see below) of 'Systematic Reviews in Health Care' (p 303-306),
* using the same dataset.

*                           Table 15.6
*  Method         Estimate (95% CI)        Sig of effect     Heterogeneity
* ---------------------------------------------------------------------------
*  Fixed effect models:
*    Cohen's d   -0.16 (-0.41 to 0.08)     z=1.29 (p=0.20)   X=2.34 (p=0.31)
*    Hedges' g   -0.16 (-0.41 to 0.08)     z=1.29 (p=0.20)   X=2.31 (p=0.32)
*    Glass's d   -0.16 (-0.40 to 0.09)     z=1.24 (p=0.21)   X=2.28 (p=0.32)

*  Random effects models:
*    Cohen's d   -0.15 (-0.42 to 0.12)     z=1.12 (p=0.26)
*    Hedges' g   -0.15 (-0.42 to 0.11)     z=1.12 (p=0.26)
*    Glass's d   -0.15 (-0.42 to 0.12)     z=1.10 (p=0.27)
* ---------------------------------------------------------------------------

* Dataset for Table 15.6  *.
DATA LIST LIST /trial(F4.0) year(A5) study(A13) n1(F8.0) mean1(Comma8.2) sd1(Comma8.2)
 n2(F8.0) mean2(Comma8.2) sd2(Comma8.2).
BEGIN DATA
1 1996 Audini-London  30  41.4 14.0 28  42.3 12.4
2 1997 Morse-StLouis  37  0.95 0.76 35  0.89 0.65
3 1998 Lehman-Baltim  67 -4.10 0.83 58 -3.80 0.87
END DATA.
VAR LABEL trial 'Trial ID number' /year 'Publication year' /study 'Trial ID name'
 /n1 'Total N ACT group' /mean1 'Mean of mental state in ACT group'
 /sd1 'Std dev of ACT group' /n2 'Total N control group'
 /mean2 'Mean of mental state in control group' /sd2 'Std dev of control group'.

* MACRO calls *.
METACONT method=2 data=mean1 sd1 n1 mean2 sd2 n2.
METACONT method=3 data=mean1 sd1 n1 mean2 sd2 n2.
METACONT method=4 data=mean1 sd1 n1 mean2 sd2 n2.
METACONT method=6 data=mean1 sd1 n1 mean2 sd2 n2.
METACONT method=7 data=mean1 sd1 n1 mean2 sd2 n2.
METACONT method=8 data=mean1 sd1 n1 mean2 sd2 n2.

* To test methods 1 & 5 (unstandardised mean differences), use the following dataset *.
DATA LIST LIST /trial(F4.0) year(A5) study(A10) n1(F8.0) mean1(Comma8.2) sd1(Comma8.2)
 n2(F8.0) mean2(Comma8.2) sd2(Comma8.2).
BEGIN DATA
1 1998	Avdeev   27 24.00 6.00 29 28.00 6.00
2 1995	Brochard 39 28.00 8.00 42 33.00 7.00
3 1998	Celikel  15 24.00 6.00 12 30.00 5.00
4 2002	Dikensoy 17 26.00 6.60 17 31.00 6.20
5 2000	Plant    92 25.45 5.39 90 27.19 5.39
END DATA.
VAR LABEL trial 'Trial ID number' /year 'Publication year' /study 'Trial ID name'
 /n1 'Total N treated (NPPV) group' /mean1 'Mean of treated (NPPV) group'
 /sd1 'Std dev of treated (NPPV) group' /n2 'Total N control group'
 /mean2 'Mean of control group' /sd2 'Std dev of control group'.
DOCUMENT 'Lightowler et al, BMJ 2003;326:185 Non-invasive positive pressure
          ventilation to treat respiratory failure resulting from exacerbations
          of chronic obstructive pulmonary disease: Cochrane systematic review
          and meta-analysis (Fig 5)'.

* MACRO calls *.
METACONT method=1 data=mean1 sd1 n1 mean2 sd2 n2.
METACONT method=5 data=mean1 sd1 n1 mean2 sd2 n2.


