PRESERVE.
SET PRINTBACK=ON.
*******************************************************
* Data: 2 variables with pvalues (1-tailed) & weights *
* (sample sizes usually, or quality scores; compute   *
* weight=1 if no sample sizes are used); the order of *
* the variables is as indicated (pvalues first)       *
*******************************************************
* Split: is an optional variable to perform subgroup  *
* analysis                                            *
*******************************************************
* The MACRO will always look for a string variable    *
* called "study" with the study names                 *
*******************************************************
* (C) Marta Garca-Granero 08/2006                    *
* send questions to: biostatistics@terra.es           *
*******************************************************.
RESTORE.

DEFINE METAP(data=!TOKENS(2)/split=!DEFAULT(NO)!TOKENS(1)).
COMPUTE @xxxx=$SYSMIS. /* Flags added variables *.
DO REPEAT A=@pvalues @weights /B=!data.
- COMPUTE A=B.
END REPEAT.
VAR LABEL @weights'Total sample size'.
COMPUTE @id=$casenum.
COMPUTE zscore=IDF.NORMAL(1-@pvalues,0,1).
!IF (!UPCASE(!split) !EQ 'NO') !THEN
- COMPUTE status=0.
!ELSE
- COMPUTE status=1.
- SORT CASES BY !UNQUOTE(!split) (A).
- FREQUENCIES VAR=!split.
!IFEND.
MATRIX.
PRINT  /TITLE '         META-ANALYSIS: COMBINATION OF ONE-TAILED P-VALUES'.
* Get data *.
GET trial  /VAR=study.
GET pvals  /VAR=@pvalues.
GET wi     /VAR=@weights.
GET zscore /VAR=zscore.
GET status /VAR=status.
COMPUTE k=NROW(pvals).
COMPUTE totaln=MSUM(wi).
COMPUTE meanz=MSUM(zscore)/k.
COMPUTE het_chi=MSUM((zscore-meanz)&**2).
COMPUTE het_sig=1-CHICDF(het_chi,k-1).
DO IF het_chi GT (k-1).
- COMPUTE isqr=100*(het_chi-(k-1))/het_chi.
- COMPUTE h=SQRT(het_chi/(k-1)).
- DO IF het_chi GT k.
-  COMPUTE eeh=LN(h)/(SQRT(2*het_chi)-SQRT(2*k-3)).
- ELSE IF het_chi LE k.
-  COMPUTE eeh=SQRT((1-(1/(3*(k-2)**2)))/(2*(k-2))).
- END IF.
- COMPUTE lowh=h*EXP(-1.96*eeh).
- COMPUTE upph=h*EXP(1.96*eeh).
- COMPUTE lowisqr=100*(lowh**2-1)/(lowh**2).
- DO IF lowisqr LT 0.
-  COMPUTE lowisqr=0.
- END IF.
- COMPUTE uppisqr=100*(upph**2-1)/(upph**2).
- DO IF uppisqr GT 100.
-  COMPUTE uppisqr=100.
- END IF.
END IF.
* Fisher's, Stouffer's & Mudholkhar-George's methods *.
COMPUTE fish_chi=-2*MSUM(LN(pvals)).
COMPUTE fish_sig=1-CHICDF(fish_chi,2*k).
COMPUTE c_term=SQRT((15*k+12)/((5*k+2)*k*3.14159265**2)).
COMPUTE mg_t=-c_term*MSUM(LN(pvals/(1-pvals))).
COMPUTE mgpvalue=1-TCDF(mg_t,5*k+4).
COMPUTE mgpvalu2=2*mgpvalue.
DO IF mgpvalu2 GT 1.
- COMPUTE mgpvalu2=1.
END IF.
COMPUTE stoufz=MSUM(zscore)/SQRT(k).
COMPUTE pvalue=1-CDFNORM(stoufz).
COMPUTE pvalue2=2*pvalue.
DO IF pvalue2 GT 1.
- COMPUTE pvalue2=1.
END IF.
COMPUTE eff_r=stoufz/SQRT(totaln).
COMPUTE besd1l=0.5-(eff_r/2).
COMPUTE besd1u=0.5+(eff_r/2).
COMPUTE wstoufz=MSUM(wi&*zscore)/SQRT(MSUM(wi&**2)).
COMPUTE wpvalue=1-CDFNORM(wstoufz).
COMPUTE wpvalue2=2*wpvalue.
DO IF wpvalue2 GT 1.
- COMPUTE wpvalue2=1.
END IF.
COMPUTE eff_wr=wstoufz/SQRT(totaln).
COMPUTE besd2l=0.5-(eff_wr/2).
COMPUTE besd2u=0.5+(eff_wr/2).
COMPUTE wrstoufz=MSUM(SQRT(wi)&*zscore)/SQRT(MSUM(wi)).
COMPUTE wrpvalue=1-CDFNORM(wrstoufz).
COMPUTE wrpvalu2=2*wrpvalue.
DO IF wrpvalu2 GT 1.
- COMPUTE wrpvalu2=1.
END IF.
COMPUTE eff_wrr=wrstoufz/SQRT(totaln).
COMPUTE besd3l=0.5-(eff_wrr/2).
COMPUTE besd3u=0.5+(eff_wrr/2).
* Fail safe N methods *.
COMPUTE fsn5=(MSUM(zscore/1.645))**2-k.
DO IF fsn5 LE 0.
- COMPUTE fsn5=0.
END IF.
COMPUTE fsn1=(MSUM(zscore/2.326))**2-k.
DO IF fsn1 LE 0.
- COMPUTE fsn1=0.
END IF.
COMPUTE a=0.01177.
COMPUTE b=-0.217*MSUM(zscore)-2.70554.
COMPUTE c=(MSUM(zscore))**2-2.70554*k.
COMPUTE fsnwc5=(-b-SQRT(b**2-4*a*c))/(2*a).
DO IF fsnwc5 le 0.
- COMPUTE fsnwc5=0.
END IF.
COMPUTE a=0.0007236.
COMPUTE b=-0.0538*MSUM(zscore)-5.4119.
COMPUTE c=(MSUM(zscore))**2-5.4119*k.
COMPUTE fsnwc1=(-b-SQRT(b**2-4*a*c))/(2*a).
DO IF fsnwc1 le 0.
- COMPUTE fsnwc1=0.
END IF.
********************** REPORTS **********************.
DO IF status(1) EQ 0. /* Long report only if no split group *.
PRINT {pvals,zscore}
 /FORMAT="F8.4"
 /CLABEL='P value','Z Score'
 /TITLE="One-tailed P-values & Z Scores to be combined"
 /RNAMES=trial.
PRINT k
 /FORMAT="F8.0"
 /TITLE="Number of trials analysed (K)".
PRINT totaln
 /FORMAT="F8.0"
 /TITLE="Total sample size (N)".
PRINT {het_chi,het_sig}
 /FORMAT="F8.4"
 /CLABELS="Value","Sig."
 /TITLE="Heterogeneity Chi-square (df=K-1). H0: Homogeneity".
DO IF het_chi GT (k-1).
- DO IF het_sig GT 0.10.
-  PRINT /TITLE="WARNING: Q p>0.10, but some heterogeneity exists!".
- END IF.
- PRINT {isqr,lowisqr,uppisqr}
 /FORMAT="F8.1"
 /CLABELS='I^2(%)','Low95 CI','Upp95 CI'
 /TITLE='Heterogeneity statistic: 25%(low), 50%(moderate), 75%(high)'.
END IF.
PRINT {fish_chi,fish_sig,(2*fish_sig)}
 /FORMAT="F8.4"
 /CLABELS="Chi2(2K)","1-tail P","2-tail P"
 /TITLE="FISHER's method (based on the product of the P values)".
PRINT {mg_t,mgpvalue,mgpvalu2}
 /FORMAT="F8.4"
 /CLABELS="t(5K+4)","1-tail P","2-tail P"
 /TITLE="MUDHOLKAR & GEORGE's method (based on logits of P values)".
PRINT /TITLE="STOUFFER's method (based on Z values)".
PRINT {stoufz,pvalue,pvalue2,eff_r,besd1l,besd1u}
 /FORMAT="F8.4"
 /CLABELS="Z","1-tail P","2-tail P","ES(r)","BESD(L)","BESD(U)"
 /TITLE="  * Unweighted".
DO IF totaln NE k.
- PRINT {wstoufz,wpvalue,wpvalue2,eff_wr,besd2l,besd2u}
 /TITLE="  * Weighted by weighting factors"
 /FORMAT="F8.4"
 /CLABELS="Z","1-tail P","2-tail P","ES(r)","BESD(L)","BESD(U)".
- PRINT {wrstoufz,wrpvalue,wrpvalu2,eff_wrr,besd3l,besd3u}
 /TITLE="  * Weighted by square roots of weighting factors"
 /FORMAT="F8.4"
 /CLABELS="Z","1-tail P","2-tail P","ES(r)","BESD(L)","BESD(U)".
END IF.
DO IF totaln LE k.
- PRINT /TITLE="No sample sizes were given, ignore ES(r) & BESD".
END IF.
DO IF fsn5 GT 0.
- PRINT /TITLE="FAIL-SAFE N (for Stouffer's Unweighted 1-tail P):".
- PRINT fsn5
 /TITLE="  * Number of non-significant results needed to push P above 0.05"
 /FORMAT="F10.0".
- PRINT fsnwc5
 /TITLE="  * Worst case calculation"
 /FORMAT="F10.0".
- PRINT fsn1
 /TITLE="  * Number of non-significant results needed to push P above 0.01"
 /FORMAT="F10.0".
- PRINT fsnwc1
 /TITLE="  * Worst case calculation"
 /FORMAT="F10.0".
END IF.
********** SENSITIVITY ANALYSIS (EFFECT OF DELETING ONE TRIAL) **********.
COMPUTE ks=k-1.
COMPUTE het_chis= MAKE(k,1,0).
COMPUTE isquares= MAKE(k,1,0).
COMPUTE lowisqus= MAKE(k,1,0).
COMPUTE higisqus= MAKE(k,1,0).
COMPUTE eff_rs=   MAKE(k,1,0).
COMPUTE besdlows= MAKE(k,1,0).
COMPUTE besdlhis= MAKE(k,1,0).
COMPUTE pvalue2s= MAKE(k,1,0).
COMPUTE dname=trial(1).
LOOP i=1 TO k. /* Extracting jack knife samples *.
- DO IF i=1.
-  COMPUTE wis=wi(2:k).
-  COMPUTE zscores=zscore(2:k).
- ELSE IF i=k.
-  COMPUTE wis=wi(1:(k-1)).
-  COMPUTE zscores=zscore(1:(k-1)).
-  COMPUTE dname={dname;trial(i)}.
- ELSE. /* (i=2 TO k-1) *.
-  COMPUTE wis={wi(1:(i-1));wi((i+1):k)}.
-  COMPUTE zscores={zscore(1:(i-1));zscore((i+1):k)}.
-  COMPUTE dname={dname;trial(i)}.
- END IF.
- COMPUTE totalns=MSUM(wis).
- COMPUTE meanzs=MSUM(zscores)/ks.
- COMPUTE het_chit=MSUM((zscores-meanzs)&**2).
- DO IF het_chit GT (ks-1).
-  COMPUTE isqr=100*(het_chit-(ks-1))/het_chit.
-  COMPUTE h=SQRT(het_chit/(ks-1)).
-  DO IF het_chit GT ks.
-   COMPUTE eeh=LN(h)/(SQRT(2*het_chit)-SQRT(2*ks-3)).
-  ELSE IF het_chit LE ks.
-   COMPUTE eeh=SQRT((1-(1/(3*(kj-2)**2)))/(2*(kj-2))).
-  END IF.
-  COMPUTE lowh=h*EXP(-1.96*eeh).
-  COMPUTE upph=h*EXP( 1.96*eeh).
-  COMPUTE lowisqr=100*(lowh**2-1)/(lowh**2).
-  DO IF lowisqr LT 0.
-   COMPUTE lowisqr=0.
-  END IF.
-  COMPUTE uppisqr=100*(upph**2-1)/(upph**2).
-  DO IF uppisqr GT 100.
-   COMPUTE uppisqr=100.
-  END IF.
- ELSE.
-  COMPUTE isqr=0.
-  COMPUTE lowisqr=0.
-  COMPUTE uppisqr=0.
- END IF.
* Stouffer's method *.
- COMPUTE stoufz=MSUM(zscores)/SQRT(ks).
- COMPUTE pvalue=1-CDFNORM(stoufz).
- COMPUTE pvalue2=2*pvalue.
- DO IF pvalue2 GT 1.
-  COMPUTE pvalue2=1.
- END IF.
- COMPUTE eff_r=stoufz/SQRT(totalns).
- COMPUTE besd1l=0.5-(eff_r/2).
- COMPUTE besd1u=0.5+(eff_r/2).
* Store summaries for report later *.
- COMPUTE het_chis(i)=het_chi-het_chit.
- COMPUTE isquares(i)=isqr.
- COMPUTE lowisqus(i)=lowisqr.
- COMPUTE higisqus(i)=uppisqr.
- COMPUTE eff_rs(i)=  eff_r.
- COMPUTE besdlows(i)=besd1l.
- COMPUTE besdlhis(i)=besd1l.
- COMPUTE pvalue2s(i)=pvalue2.
END LOOP.
PRINT /TITLE='SENSITIVITY ANALYSIS: RESULTS DELETING ONE TRIAL EACH TIME'.
PRINT {het_chis,isquares,lowisqus,higisqus,eff_rs,besdlows,besdlhis,pvalue2s}
 /FORMAT='F8.3'
 /CLABELS=' Change*','I-square','95%Lower','95%Upper','ES(r)','BESD(L)','BESD(U)','2-tail P'
 /RNAME=dname
 /TITLE='          Het.Chi                              <- From unweighted Stouffer Z ->'.
PRINT /TITLE=' (*) Change in heterogeneity Chi-square after deleting trial (Sig if Chi>3.84)'.
********** REPEAT ANALYSIS BY SUBGROUPS (IF SPLITVAR IS INCLUDED) **********.
ELSE IF (status(1) EQ 1). /* Condensed report by subgroups *.
GET splitvar /VAR=!split /NAMES=varname.
PRINT varname
/FORMAT='A8'
 /TITLE='SUBGROUP ANALYSIS, SPLITTING FOR:'.
COMPUTE codes=(CSUM(T(splitvar)*DESIGN(splitvar)))/CSUM(DESIGN(splitvar)).
COMPUTE nbycode=CSUM(DESIGN(splitvar)).
COMPUTE nsplits=  NCOL(nbycode).
COMPUTE het_chig= MAKE(nsplits,1,0).
COMPUTE isquarej= MAKE(nsplits,1,0).
COMPUTE lowisquj= MAKE(nsplits,1,0).
COMPUTE higisquj= MAKE(nsplits,1,0).
COMPUTE eff_rj=   MAKE(nsplits,1,0).
COMPUTE besdlowj= MAKE(nsplits,1,0).
COMPUTE besdlhij= MAKE(nsplits,1,0).
COMPUTE pvalue2j= MAKE(nsplits,1,0).
COMPUTE count=1.
LOOP j=RMIN(codes) TO RMAX(codes).
- COMPUTE wij=    MAKE(nbycode(j),1,0).
- COMPUTE zscorej=MAKE(nbycode(j),1,0).
- LOOP k=1 TO nbycode(j).
-  COMPUTE wij(k)=    wi(count).
-  COMPUTE zscorej(k)=zscore(count).
-  COMPUTE count=     count+1.
- END LOOP.
- COMPUTE kj=NROW(zscorej).
- COMPUTE totalnj=MSUM(wij).
- COMPUTE meanzj=MSUM(zscorej)/kj.
- COMPUTE het_chij=MSUM((zscorej-meanzj)&**2).
- COMPUTE het_sigj=1-CHICDF(het_chij,kj-1).
- DO IF (het_chij GT (kj-1) AND (kj GT 2)).
-  COMPUTE isqr=100*(het_chij-(kj-1))/het_chij.
-  COMPUTE h=SQRT(het_chij/(kj-1)).
-  DO IF het_chij GT kj.
-   COMPUTE eeh=LN(h)/(SQRT(2*het_chij)-SQRT(2*kj-3)).
-  ELSE IF het_chij LE kj.
-   COMPUTE eeh=SQRT((1-(1/(3*(kj-2)**2)))/(2*(kj-2))).
-  END IF.
-  COMPUTE lowh=h*EXP(-1.96*eeh).
-  COMPUTE upph=h*EXP( 1.96*eeh).
-  COMPUTE lowisqr=100*(lowh**2-1)/(lowh**2).
-  DO IF lowisqr LT 0.
-   COMPUTE lowisqr=0.
-  END IF.
-  COMPUTE uppisqr=100*(upph**2-1)/(upph**2).
-  DO IF uppisqr GT 100.
-   COMPUTE uppisqr=100.
-  END IF.
- ELSE.
-  COMPUTE isqr=0.
-  COMPUTE lowisqr=0.
-  COMPUTE uppisqr=0.
- END IF.
* Stouffer's method *.
- COMPUTE stoufz=MSUM(zscorej)/SQRT(kj).
- COMPUTE pvalue=1-CDFNORM(stoufz).
- COMPUTE pvalue2=2*pvalue.
- DO IF pvalue2 GT 1.
-  COMPUTE pvalue2=1.
- END IF.
- COMPUTE eff_r=stoufz/SQRT(totalnj).
- COMPUTE besd1l=0.5-(eff_r/2).
- COMPUTE besd1u=0.5+(eff_r/2).
* Store summaries for report later *.
- COMPUTE het_chig(j)=het_chij.
- COMPUTE isquarej(j)=isqr.
- COMPUTE lowisquj(j)=lowisqr.
- COMPUTE higisquj(j)=uppisqr.
- COMPUTE eff_rj(j)=eff_r.
- COMPUTE besdlowj(j)=besd1l.
- COMPUTE besdlhij(j)=besd1l.
- COMPUTE pvalue2j(j)= pvalue2.
END LOOP.
COMPUTE totalchi=MSUM(het_chig).
COMPUTE totchisi=1-CHICDF(totalchi,k-nsplits).
COMPUTE betchi=het_chi-totalchi.
COMPUTE betchisi=1-CHICDF(betchi,nsplits-1).
PRINT {het_chig,isquarej,lowisquj,higisquj,eff_rj,besdlowj,besdlhij,pvalue2j}
 /FORMAT='F8.3'
 /CLABELS='Het.Chi','I-square','95%Lower','95%Upper','ES(r)','BESD(L)','BESD(U)','2-tail P'
 /TITLE='Statistics by split group               <- From unweighted Stouffer Z ->'.
PRINT {betchi,betchisi}
 /FORMAT='F8.4'
 /CLABELS='Value','Sig.'
 /TITLE='Between Groups Heterogeneity Chi-square (df=N Grps-1). H0: Homogeneity'.
END IF.
END MATRIX.
* Restoring dataset *.
SORT CASES BY @id(A).
COMPUTE @yyyy=$SYSMIS.
MATCH FILES /FILE=* /DROP= @xxxx TO @yyyy.
!ENDDEFINE.

* Example dataset *.
DATA LIST LIST /trial(F4.0) year(A5) study(A10) monoter(F4.0) pvals(Comma8.4)
 n1(F8.0) n2(F8.0).
BEGIN DATA
 1 1994 Au         1 0.5000   26  24
 2 1992 Corneliss  1 0.6057   47  47
 3 1998 Dincol     1 0.7729   71  72
 4 1994 Kojima     1 0.6299   36  34
 5 1992 Leyland    1 0.0003  164 148
 6 1990 Lieschke   1 0.9355   90  92
 7 1989 Liu        1 0.9021   10  18
 8 1987 Norrby     1 0.8106  105 105
 9 1999 Ozyilkan   1 0.7586   15  15
10 1995 Perez      1 0.2433   30  30
11 2001 Agaoglu    2 0.9617   30  57
12 1999 Akove      2 0.4582   40  43
13 1998 Behre      2 .00004   39  39
14 1996 Cornetta   2 0.5422  516 511
15 1997 DelaCamara 2 0.4176   62  60
16 1983 DePauw-1   3 0.5262   42  45
17 1994 DePauw-2   3 1E-06   551 535
18 1983 Doyen      3 0.2841   50  54
19 1989 Gibson     3 0.0009   52  50
20 1993 Jacobs     3 0.1180   53  54
21 1991 Marie      3 0.4147   77  69
22 1983 Morgan     3 0.4800   26  24
23 1991 Novakova   3 0.6211   45  45
24 1988 Piguet     3 0.3934   82  87
25 1983 Alanis     4 0.1536   53  55
26 1985 Bezwoda    4 0.0545   29  31
27 1988 Hansen     4 0.5000   14  19
28 1984 Pegram     4 0.0691   72  68
29 1983 Pickard    4 0.5000   40  40
30 1998 Hess       5 0.6677   48  48
31 1984 Piccart    5 0.5000   25  24
32 1990 Smith      5 0.4546   47  53
33 2002 Tamura     5 0.9410   77  76
34 1997 Yamamura   5 0.0090   56  55
END DATA.
VAR LABEL trial 'Trial ID number' /year 'Publication year' /study 'Trial ID name'
 /monoter 'Monotherapy used' /pvals 'One tailed p-values'
 /n1 'Total N treated group' /n2 'Total N control group'.
VALUE LABELS monoter  1 'Imipenem' 2 'Meropenem' 3 'Ceftazidime'
 4 'Moxalactan' 5 'Other monoth.'.

DOCUMENT 'Paul et al, BMJ 2003;326:1111  lactam monotherapy versus  lactam
          -aminoglycoside combination therapy for fever with neutropenia:
          systematic review and meta-analysis (Fig 5)'.

COMPUTE nt=n1+n2.
EXECUTE.
FORMAT nt (F8.0).

* MACRO call without subgroup analysis *.
METAP data=pvals nt.

* MACRO call with monoter as group variable *.
METAP data=pvals nt split=monoter.
/* Or: METAP split=monoter data=pvals nt. *.
