begin program python.
import spss
from spssaux import *
from bootstrap_object import *

bootstrap_iterations = 10

# running time will be proportional to the number of bootstrap iterations
# plus the number of cases, when BCa confidence intervals are requested

test_data = MakeDataset(dataset="created_data_", cases=30)
test_data.addCompute("a", "rv.normal(0,1)", "Alice")
test_data.addCompute("b", "rv.normal(0,1)", "Bob")
test_data.addCompute("c", "rv.normal(0,1)", "Charlie")
test_data.addCompute("d", "rv.normal(a,1)", "Dennis")
test_data.addCompute("y", "1 + 2*a - 3*b + 4*c + 1*d + rv.normal(0,1)", "Dependent")
data = test_data.Submit()
 
depvar = "y"
indvars = ["a", "b", "c", "d"]
variables = indvars + [depvar]
#variables = test_data.variables

syntax = "Regression /DEPENDENT %s /METHOD=ENTER %s ." % (depvar, " ".join(indvars))

### ... Nothing changes below, except the values requested ...

# data to be bootstrapped should not have missing values;
# select only the complete cases
spss.Submit(selectCompleteCases(variables))

varDict = VariableDict()

# set up a dictionary of variable labels indexed by variable names
# later, the labels will be used to retrieve results from tables
vlabels = {}
for i in range(varDict.VariableCount()):
    vlabels[varDict.VariableName(i)] = varDict.VariableLabel(i)

# get ready to retrieve results from Regression Coefficients table 
oms = OMS(commands="Regression", subtypes="Coefficients", viewer=True)
# ask for the row labelled (Constant) in the column labelled B
# the result will be kept in a dictionary indexed by constant
oms.requestValues("constant", rowCategory="(Constant)", colCategory="B")
oms.requestRows(vlabels, indvars, colCategory="B")
job = Job(syntax, oms)

# in case we want to know what the OMS commands actually were...
#print oms.commands()

# this time don't ask for the output in the viewer
oms = OMS(commands="Regression", subtypes="Coefficients")
oms.requestValues("constant", rowCategory="(Constant)", colCategory="B")
oms.requestRows(vlabels, indvars, colCategory="B")

job = Bootstrap(syntax, oms, iterations=bootstrap_iterations)
bootrep = job.Submit()
try:
    job.bcaSummary(useCTables=True)
except:
    print "Sorry, CTables output would look much better"
job.bcaSummary(useCTables=False)

print "Sorry, that's all for today."

# cleanup time - any unsaved datasets will disappear
for oms in OMS.activeRequests.iteritems():
    print "Closing OMS %s" % oms.tag
    oms.close()
for data in Dataset.datasets.copy().itervalues():
    print "Closing dataset %s" % data.dataset
    data.close()
end program.
